/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import javax.microedition.pim.PIM;
import javax.microedition.pim.PIMList;
import net.rim.blackberry.api.pdap.BlackBerryPIM;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.device.api.system.Application;

import com.funambol.blackberry.helper.EventMailListener;
import com.funambol.blackberry.helper.HelperFactory;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.blackberry.source.BBAppSyncSource;
import com.funambol.blackberry.source.BBAppSyncSourceConfig;
import com.funambol.blackberry.source.pim.BBPIMSyncSource;
import com.funambol.blackberry.source.pim.BBPIMSyncSourceWithUpdateFix;
import com.funambol.blackberry.source.media.picture.PictureSyncSource;
import com.funambol.blackberry.source.media.picture.TwoPhasesPictureSyncSource;
import com.funambol.blackberry.source.media.MediaAppSyncSourceConfig;
import com.funambol.blackberry.source.media.picture.PictureAppSyncSourceConfig;
import com.funambol.blackberry.source.media.video.VideoAppSyncSourceConfig;
import com.funambol.blackberry.source.media.MediaAppSyncSource;
import com.funambol.blackberry.source.pim.BBPIMListener;
import com.funambol.blackberry.source.media.picture.PictureAppSyncSource;
import com.funambol.blackberry.source.media.video.VideoAppSyncSource;
import com.funambol.blackberry.source.pim.calendar.CalendarAppSyncSourceConfig;

import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.client.ConfigSyncSource;
import com.funambol.syncml.client.CacheTracker;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.storage.StringKeyValueFileStore;
import com.funambol.client.source.AppSyncSource;
import com.funambol.client.source.ExternalAppManager;
import com.funambol.client.source.AppSyncSourceManager;
import com.funambol.client.source.AppSyncSourceConfig;
import com.funambol.util.BlackberryUtils;
import com.funambol.util.StringUtil;
import com.funambol.util.Log;


/**
 * This class is responsible for handling all the sources at the application
 * level. A source at the application level is a type of data for which a
 * SyncSource exits, but also a sync button on the screen and configuration
 * parameters.
 * This class creates and registers the sources. The list of sources is handled
 * and allows other parts of the application to be completely parametric wrt to
 * this value.
 * For new sources to be added to the application it is necessary to register
 * them in this manager. The only other part of the application that must be
 * changed when new sources are added is the BBCustomization which holds default
 * parameters for each source.
 */
public class BBAppSyncSourceManager extends AppSyncSourceManager {

    private static final String TAG_LOG = "BBAppSyncSourceManager";

    public static final int     SYNC_FLAG_CONTACTS        = CONTACTS_ID;
    public static final int     SYNC_FLAG_CALENDAR        = EVENTS_ID;
    public static final int     SYNC_FLAG_TASKS           = TASKS_ID;
    public static final int     SYNC_FLAG_NOTES           = NOTES_ID;
    public static final int     SYNC_FLAG_MAIL            = MAILS_ID;
    public static final int     SYNC_FLAG_PHOTOS          = PICTURES_ID;
    public static final int     SYNC_FLAG_CONFIG          = CONFIG_ID;
    public static final int     SYNC_FLAG_VIDEOS          = VIDEOS_ID;
    public static final int     SYNC_FLAG_ALL             = ALL_ID;

    // Configuration keys for the source configs
    public static final String  CONFIG_KEY_SYNC_CONTACT   = "SYNC_CONTACT_CONFIG";
    public static final String  CONFIG_KEY_SYNC_CALENDAR  = "SYNC_CAL_CONFIG";
    public static final String  CONFIG_KEY_SYNC_TASK      = "SYNC_TASK_CONFIG";
    public static final String  CONFIG_KEY_SYNC_NOTE      = "SYNC_NOTE_CONFIG";
    public static final String  CONFIG_KEY_SYNC_PHOTO     = "SYNC_PHOTO_CONFIG";
    public static final String  CONFIG_KEY_SYNC_VIDEO     = "SYNC_VIDEO_CONFIG";
    public static final String  CONFIG_KEY_SYNC_CONFIG    = "SYNC_CONFIG_CONFIG";

    public static final long    CONTACT_STORAGE_KEY       = 0xbc565a75eda5330L;
    public static final long    CALENDAR_STORAGE_KEY      = 0xbc565a75eda5340L;
    public static final long    NOTE_STOREAGE_KEY         = 0xbc565a75eda5350L;
    public static final long    TASK_STORAGE_KEY          = 0xbc565a75eda5360L;
    public static final long    PHOTO_STORAGE_KEY         = 0xbc565a75eda5380L;
    public static final long    VIDEO_STORAGE_KEY         = 0xbc565a75eda53B0L;
    public static final long    CONFIG_STORAGE_KEY        = 0xbc565a75eda53A0L;

    public static final long    CONTACTS_ADDITS_STORAGE_KEY = 0xbc565a75eda5390L;
    public static final long    CALENDAR_ADDITS_STORAGE_KEY = 0xbc565a75eda5370L;
    public static final long    NOTE_ADDITS_STORAGE_KEY     = -1;
    public static final long    TASKS_ADDITS_STORAGE_KEY    = -1;

    private static final String CONFIG_SOURCE_NAME        = "config";


    private static BBAppSyncSourceManager instance = null;
    private BBConfiguration configuration = null;
    private BBLocalization  localization = BBLocalization.getInstance();

    private BBFileSystemListener fileSystemListener = null;

    private boolean initialized = false;

    private BBAppSyncSourceManager() {
        super(BBCustomization.getInstance());
    }

    public static BBAppSyncSourceManager getInstance() {
        if (instance == null) {
            instance = new BBAppSyncSourceManager();
        }
        return instance;
    }

    private void init() {
        if (!initialized) {
            configuration = BBConfiguration.getInstance();
            // Register the file system journal listener
            fileSystemListener = new BBFileSystemListener(BBConfiguration.getInstance(), Log.getInstance());
            Application app = Application.getApplication();
            app.addFileSystemJournalListener(fileSystemListener);

            initialized = true;
        }
    }

    public void setUpContactsSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "Setting up Contacts source");

        init();

        // Build the appropriate AppSyncSource and register it, even if the
        // corresponding SyncSource has not been built yet
        String name = localization.getLanguage("type_contacts");
        BBAppSyncSource contactSource = new BBAppSyncSource(name);
        contactSource.setId(SYNC_FLAG_CONTACTS);
        contactSource.setEnabledLabel(localization.getLanguage("conf_enable-sync-contact"));
        contactSource.setDisabledLabel(localization.getLanguage("message_contacts-disabled"));
        contactSource.setIconName((String)BBCustomization.getInstance().getSourceIcon(SYNC_FLAG_CONTACTS).getOpaqueDescriptor());
        contactSource.setDisabledIconName((String)BBCustomization.getInstance().getSourceDisabledIcon(SYNC_FLAG_CONTACTS).getOpaqueDescriptor());
        contactSource.setUiSourceIndex(uiOrder);
        contactSource.setConfigStorageKey(CONFIG_KEY_SYNC_CONTACT);
        contactSource.setHasSettings(true);
        Class settingsClass = Class.forName("com.funambol.blackberry.source.pim.contact.ContactSourceSettings");
        contactSource.setSettingsUIClass(settingsClass);
        contactSource.setIsMedia(false);
        registerSource(contactSource);

        String contactType = customization.getContactType();
        boolean base64 = !CONTACT_TYPE_VCARD.equals(contactType);
        String defaultUri = customization.getDefaultSourceUri(SYNC_FLAG_CONTACTS);

        ConfigPair configs = initPIMSourceConfig(contactSource, name, contactType, defaultUri, base64);
        BBAppSyncSourceConfig asc = configs.appSyncSourceConfig;
        SourceConfig        config = configs.sourceConfig;
        contactSource.setConfig(asc);

        boolean updateFix = false;
        String version[] = BlackberryUtils.getSplittedOSVersion();
        if (version.length > 1 && version[0].equals("4") && version[1].equals("5")) {
            Log.debug(TAG_LOG, "Version is 4.5, use source with update fix");
            updateFix = true;
        } else { 
            Log.debug(TAG_LOG, "Version is NOT 4.5, use standard source"); 
        }

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.CONTACT_LIST, PIM.READ_WRITE);

        SyncSource syncSource;
        if (updateFix) {
            syncSource = new BBPIMSyncSourceWithUpdateFix(config, list,
                               HelperFactory.getContactHelper(list),
                               CONTACT_STORAGE_KEY,
                               CONTACTS_ADDITS_STORAGE_KEY);
        } else {
            syncSource = new BBPIMSyncSource(config, list,
                                    HelperFactory.getContactHelper(list),
                                    CONTACT_STORAGE_KEY,
                                    CONTACTS_ADDITS_STORAGE_KEY);
        }

        syncSource.setConfig(config);
        // Reset the underlying sync source
        contactSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new BBExternalAppManager(contactSource);
        contactSource.setAppManager(appManager);
        Log.debug(TAG_LOG, "Initializing config (5)");
        // Register the event listener for this source
        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            BBPIMSyncSource   bbSource = (BBPIMSyncSource)syncSource;
            BBPIMListener listener = new BBPIMListener(bbList, contactSource,configuration,Log.getInstance());
            bbSource.setPIMListener(listener);
            bbList.addListener(listener);
            contactSource.setNativeListener(listener);
        }
    }

    public void setUpCalendarSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "Setting up Calendar source");

        init();

        // Build the appropriate AppSyncSource
        String name = localization.getLanguage("type_calendar");
        BBAppSyncSource calendarSource = new BBAppSyncSource(name);
        calendarSource.setId(SYNC_FLAG_CALENDAR);
        calendarSource.setEnabledLabel(localization.getLanguage("conf_enable-sync-calendar"));
        calendarSource.setDisabledLabel(localization.getLanguage("message_calendar-disabled"));
        calendarSource.setIconName((String)BBCustomization.getInstance().getSourceIcon(SYNC_FLAG_CALENDAR).getOpaqueDescriptor());
        calendarSource.setDisabledIconName((String)BBCustomization.getInstance().getSourceDisabledIcon(SYNC_FLAG_CALENDAR).getOpaqueDescriptor());
        calendarSource.setUiSourceIndex(uiOrder);
        calendarSource.setConfigStorageKey(CONFIG_KEY_SYNC_CALENDAR);
        calendarSource.setHasSettings(true);
        Class settingsClass = Class.forName("com.funambol.blackberry.source.pim.calendar.CalendarSourceSettings");
        calendarSource.setSettingsUIClass(settingsClass);
        calendarSource.setIsMedia(false);
        registerSource(calendarSource);

        String calendarType = customization.getCalendarType();
        boolean base64 = calendarType.equals(CALENDAR_TYPE_SIF);
        String defaultUri = customization.getDefaultSourceUri(SYNC_FLAG_CALENDAR);

        ConfigPair configs = initPIMSourceConfig(calendarSource, name, calendarType, defaultUri, base64);
        BBAppSyncSourceConfig asc = configs.appSyncSourceConfig;
        SourceConfig        config = configs.sourceConfig;
        calendarSource.setConfig(asc);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.EVENT_LIST, PIM.READ_WRITE);

        EventMailListener listener = null;
        if (BBCustomization.getInstance().invitesFiltering()) {
            listener = new EventMailListener(localization.getLanguage("message_blocked-invite"));
        }

        int rangePast = ((CalendarAppSyncSourceConfig)asc).getRangePast();
        int rangeFuture = ((CalendarAppSyncSourceConfig)asc).getRangeFuture();
        PIMItemHelper eventhelper = HelperFactory.getEventHelper(list, listener, rangePast, rangeFuture);

        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list,
                                         eventhelper, CALENDAR_STORAGE_KEY, CALENDAR_ADDITS_STORAGE_KEY);
        syncSource.setConfig(config);

        // Reset the underlying sync source
        calendarSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new BBExternalAppManager(calendarSource);
        calendarSource.setAppManager(appManager);
        // Register the event listener for this source
        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            BBPIMSyncSource   bbSource = (BBPIMSyncSource)syncSource;

            BBPIMListener pimListener = new BBPIMListener(bbList,calendarSource,configuration,Log.getInstance());
            bbSource.setPIMListener(pimListener);
            bbList.addListener(pimListener);
            calendarSource.setNativeListener(pimListener);
        }
    }

    public void setUpTasksSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "Setting up Tasks source");

        init();

        // Build the appropriate AppSyncSource
        String name = localization.getLanguage("type_tasks");

        BBAppSyncSource tasksSource = new BBAppSyncSource(name);
        tasksSource.setId(SYNC_FLAG_TASKS);
        tasksSource.setEnabledLabel(localization.getLanguage("conf_enable-sync-task"));
        tasksSource.setDisabledLabel(localization.getLanguage("message_tasks-disabled"));
        tasksSource.setIconName((String)BBCustomization.getInstance().getSourceIcon(SYNC_FLAG_TASKS).getOpaqueDescriptor());
        tasksSource.setDisabledIconName((String)BBCustomization.getInstance().getSourceDisabledIcon(SYNC_FLAG_TASKS).getOpaqueDescriptor());
        tasksSource.setUiSourceIndex(uiOrder);
        tasksSource.setConfigStorageKey(CONFIG_KEY_SYNC_TASK);
        tasksSource.setHasSettings(true);
        Class settingsClass = Class.forName("com.funambol.blackberry.ui.rim.RimSourceBasicSettings");
        tasksSource.setSettingsUIClass(settingsClass);
        tasksSource.setIsMedia(false);
        registerSource(tasksSource);

        String taskType = customization.getTaskType();
        boolean base64 = taskType.equals(CALENDAR_TYPE_SIF);
        String defaultUri = customization.getDefaultSourceUri(SYNC_FLAG_TASKS);

        ConfigPair configs = initPIMSourceConfig(tasksSource, name, taskType, defaultUri, base64);
        BBAppSyncSourceConfig asc = configs.appSyncSourceConfig;
        SourceConfig        config = configs.sourceConfig;
        tasksSource.setConfig(asc);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.TODO_LIST,
                                                                 PIM.READ_WRITE);
        PIMItemHelper helper = HelperFactory.getTaskHelper(list);

        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list, helper, TASK_STORAGE_KEY, TASKS_ADDITS_STORAGE_KEY);

        syncSource.setConfig(config);

        tasksSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new BBExternalAppManager(tasksSource);
        tasksSource.setAppManager(appManager);
        // Register the event listener for this source
        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            BBPIMSyncSource   bbSource = (BBPIMSyncSource)syncSource;
            BBPIMListener listener = new BBPIMListener(bbList, tasksSource,configuration,Log.getInstance());
            bbSource.setPIMListener(listener);
            bbList.addListener(listener);
            tasksSource.setNativeListener(listener);
        }
    }

    public void setUpNotesSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "Setting up Notes source");

        init();

        // Build the appropriate AppSyncSource
        String name = localization.getLanguage("type_notes");
        BBAppSyncSource notesSource = new BBAppSyncSource(name);
        notesSource.setId(SYNC_FLAG_NOTES);
        notesSource.setEnabledLabel(localization.getLanguage("conf_enable-sync-note"));
        notesSource.setDisabledLabel(localization.getLanguage("message_notes-disabled"));
        notesSource.setIconName((String)BBCustomization.getInstance().getSourceIcon(SYNC_FLAG_NOTES).getOpaqueDescriptor());
        notesSource.setDisabledIconName((String)BBCustomization.getInstance().getSourceDisabledIcon(SYNC_FLAG_NOTES).getOpaqueDescriptor());
        notesSource.setUiSourceIndex(uiOrder);
        notesSource.setConfigStorageKey(CONFIG_KEY_SYNC_NOTE);
        notesSource.setHasSettings(true);
        Class settingsClass = Class.forName("com.funambol.blackberry.ui.rim.RimSourceBasicSettings");
        notesSource.setSettingsUIClass(settingsClass);
        notesSource.setIsMedia(false);
        registerSource(notesSource);

        String defaultUri = customization.getDefaultSourceUri(SYNC_FLAG_NOTES);

        ConfigPair configs = initPIMSourceConfig(notesSource, name, NOTE_TYPE, defaultUri, true);
        BBAppSyncSourceConfig asc = configs.appSyncSourceConfig;
        SourceConfig        config = configs.sourceConfig;
        notesSource.setConfig(asc);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(BlackBerryPIM.MEMO_LIST, PIM.READ_WRITE);
        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list, HelperFactory.getNoteHelper(list),
                                         NOTE_STOREAGE_KEY, NOTE_ADDITS_STORAGE_KEY);

        syncSource.setConfig(config);

        notesSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new BBExternalAppManager(notesSource);
        notesSource.setAppManager(appManager);
        // Register the event listener for this source
        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            BBPIMListener listener = new BBPIMListener(bbList, notesSource,configuration,Log.getInstance());
            BBPIMSyncSource   bbSource = (BBPIMSyncSource)syncSource;
            bbSource.setPIMListener(listener);
            bbList.addListener(listener);
            notesSource.setNativeListener(listener);
        }
    }

    public void setUpPicturesSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "Setting up Pictures source");

        init();

        // These are the supported extensions
        String extensions[] = { "jpg", "jpeg", "gif", "png" };

        String defaultDir = ((BBCustomization)customization).getDefaultPicturesDir();
        String defaultSDCardDir = ((BBCustomization)customization).getDefaultPicturesSDCardDir();

        // Build the appropriate AppSyncSource
        String name = localization.getLanguage("type_photos");
        PictureAppSyncSource photosSource = new PictureAppSyncSource(name);
        photosSource.setId(SYNC_FLAG_PHOTOS);
        photosSource.setEnabledLabel(localization.getLanguage("conf_enable-sync-photo"));
        photosSource.setDisabledLabel(localization.getLanguage("message_photos-disabled"));
        photosSource.setIconName((String)BBCustomization.getInstance().getSourceIcon(SYNC_FLAG_PHOTOS).getOpaqueDescriptor());
        photosSource.setDisabledIconName((String)BBCustomization.getInstance().getSourceDisabledIcon(SYNC_FLAG_PHOTOS).getOpaqueDescriptor());
        photosSource.setUiSourceIndex(uiOrder);
        photosSource.setConfigStorageKey(CONFIG_KEY_SYNC_PHOTO);
        photosSource.setHasSettings(true);
        String sourceName = name.toLowerCase();
        String firstSyncWarningMsg = StringUtil.replaceAll(localization.getLanguage("dialog_first_media_sync"), "__source__", sourceName);
        photosSource.setWarningOnFirstSync(firstSyncWarningMsg);
        photosSource.setExtensions(extensions);
        photosSource.setSDCardMediaDir(defaultSDCardDir);
        photosSource.setInternalMemoryMediaDir(defaultDir);
        Class settingsClass = Class.forName("com.funambol.blackberry.source.media.MediaSourceSettings");
        photosSource.setSettingsUIClass(settingsClass);
        photosSource.setIsMedia(true);
        registerSource(photosSource);

        String defaultUri = customization.getDefaultSourceUri(SYNC_FLAG_PHOTOS);
        
        // Load the app sync source config
        PictureAppSyncSourceConfig asc  = new PictureAppSyncSourceConfig(photosSource, BBCustomization.getInstance(),
                                                                         BBConfiguration.getInstance(), defaultDir);
        SourceConfig config = initMediaSourceConfig(asc, photosSource, name, PHOTO_TYPE, defaultUri, false, defaultDir);
        asc.setSyncType(SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW);
        photosSource.setConfig(asc);

        // Create the sync source (also the one needed for backward
        // compatibility)
        TwoPhasesPictureSyncSource syncSource = new TwoPhasesPictureSyncSource(asc, config, extensions,
                configuration.getSyncConfig(), BBCustomization.getInstance().getHttpUploadPrefix());

        PictureSyncSource syncMLSrc = new PictureSyncSource(asc, config, extensions);

        photosSource.setSyncMLSource(syncMLSrc);
        photosSource.setTwoPhasesSyncSource(syncSource);

        if (asc.getUploadContentViaHttp()) {
            photosSource.setSyncSource(syncSource);
        } else {
            photosSource.setSyncSource(syncMLSrc);
        }
        
        // Set the open application (used in the goto menu)
        //ExternalAppManager appManager = new ExternalAppManager(photosSource);
        //photosSource.setAppManager(appManager);
        String directory = asc.getDirectory();
        BBFileSystemListener.BBFileSystemListenerDirectory dir = fileSystemListener.new BBFileSystemListenerDirectory(photosSource, directory);
        fileSystemListener.addDirectory(dir);
        photosSource.setNativeListener(dir);
    }

    public void setUpVideosSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "Setting up Videos source");

        init();

        // These are the extensions supported by this source
        String extensions[] = { "3gp", "mp4", "mpeg", "flv" };

        String defaultDir = ((BBCustomization)customization).getDefaultVideosDir();
        String defaultSDCardDir = ((BBCustomization)customization).getDefaultVideosSDCardDir();
        
        // Build the appropriate AppSyncSource
        String name = localization.getLanguage("type_videos");
        VideoAppSyncSource videosSource = new VideoAppSyncSource(name);
        videosSource.setId(SYNC_FLAG_VIDEOS);
        videosSource.setEnabledLabel(localization.getLanguage("conf_enable-sync-photo"));
        videosSource.setDisabledLabel(localization.getLanguage("message_photos-disabled"));
        videosSource.setIconName((String)BBCustomization.getInstance().getSourceIcon(SYNC_FLAG_VIDEOS).getOpaqueDescriptor());
        videosSource.setDisabledIconName((String)BBCustomization.getInstance().getSourceDisabledIcon(SYNC_FLAG_VIDEOS).getOpaqueDescriptor());
        videosSource.setUiSourceIndex(uiOrder);
        videosSource.setConfigStorageKey(CONFIG_KEY_SYNC_VIDEO);
        videosSource.setHasSettings(true);
        String sourceName = name.toLowerCase();
        String firstSyncWarningMsg = StringUtil.replaceAll(localization.getLanguage("dialog_first_media_sync"), "__source__", sourceName);
        videosSource.setWarningOnFirstSync(firstSyncWarningMsg);
        videosSource.setExtensions(extensions);
        videosSource.setSDCardMediaDir(defaultSDCardDir);
        videosSource.setInternalMemoryMediaDir(defaultDir);
        Class settingsClass = Class.forName("com.funambol.blackberry.source.media.MediaSourceSettings");

        videosSource.setSettingsUIClass(settingsClass);
        videosSource.setIsMedia(true);
        registerSource(videosSource);

        String defaultUri = customization.getDefaultSourceUri(SYNC_FLAG_VIDEOS);

        VideoAppSyncSourceConfig asc  = new VideoAppSyncSourceConfig(videosSource, BBCustomization.getInstance(),
                                                                     BBConfiguration.getInstance(), defaultDir);
        SourceConfig config = initMediaSourceConfig(asc, videosSource, name, VIDEO_TYPE, defaultUri, false, defaultDir);
        asc.setSyncType(SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW);
        videosSource.setConfig(asc);

        // Create the sync source (also the one needed for backward
        // compatibility)
        TwoPhasesPictureSyncSource syncSource = new TwoPhasesPictureSyncSource(asc, config, extensions,
                configuration.getSyncConfig(), BBCustomization.getInstance().getHttpUploadPrefix());

        videosSource.setSyncSource(syncSource);
        
        // Set the open application (used in the goto menu)
        //ExternalAppManager appManager = new ExternalAppManager(photosSource);
        //photosSource.setAppManager(appManager);
        String directory = asc.getDirectory();
        BBFileSystemListener.BBFileSystemListenerDirectory dir = fileSystemListener.new BBFileSystemListenerDirectory(videosSource, directory);
        fileSystemListener.addDirectory(dir);
        videosSource.setNativeListener(dir);
    }


    public void setUpConfigSource() throws Exception {
        Log.info(TAG_LOG, "Setting up Configuration source");

        init();

        // Build the appropriate AppSyncSource
        String name = CONFIG_SOURCE_NAME;
        BBAppSyncSource configSource = new BBAppSyncSource(name);
        configSource.setId(SYNC_FLAG_CONFIG);
        configSource.setEnabledLabel(null);
        configSource.setDisabledLabel(null);
        configSource.setIconName(null);
        configSource.setDisabledIconName(null);
        configSource.setUiSourceIndex(0);
        configSource.setConfigStorageKey(CONFIG_KEY_SYNC_CONFIG);
        configSource.setHasSettings(false);
        configSource.setIsRefreshSupported(false);
        configSource.setIsVisible(false);
        configSource.setIsMedia(false);
        registerSource(configSource);

        String defaultUri = customization.getDefaultSourceUri(SYNC_FLAG_CONFIG);
        SourceConfig config = new SourceConfig(name, CONFIG_TYPE, defaultUri);

        // Load the app sync source config
        BBAppSyncSourceConfig asc = new BBAppSyncSourceConfig(configSource,
                BBCustomization.getInstance(), BBConfiguration.getInstance());
        asc.load(config);
        configSource.setConfig(asc);

        StringKeyValueFileStore cacheStore = new StringKeyValueFileStore("file:///store/home/user/configcache.txt");
        StringKeyValueFileStore configStore = new StringKeyValueFileStore("file:///store/home/user/configstore.txt");
        CacheTracker tracker = new CacheTracker(cacheStore);
 
        SyncSource syncSource;
        syncSource = new ConfigSyncSource(config, tracker, configStore);

        syncSource.setConfig(config);

        // Reset the underlying sync source
        configSource.setSyncSource(syncSource);
    }

    /*
     * Disable a given source. After this method returns the source is
     * disabled for syncs and the setting is propagated into the configuration
     */
    public void disableSource(int id) {
        AppSyncSource appSource = getSource(id);
        AppSyncSourceConfig config = appSource.getConfig();
        config.setEnabled(false);
        config.save();
    }

    private ConfigPair initPIMSourceConfig(AppSyncSource appSource, String name, String type, String uri, boolean base64) {
        Log.info(TAG_LOG, "Initialize source config for [" + name + "]");
        SourceConfig config = new SourceConfig(name, type, uri);
        
        // Load the app sync source config
        BBAppSyncSourceConfig asc;
        if(appSource.getId() == AppSyncSourceManager.EVENTS_ID) {
            asc = new CalendarAppSyncSourceConfig(appSource, BBCustomization.getInstance(), BBConfiguration.getInstance());
        } else {
            asc = new BBAppSyncSourceConfig(appSource, BBCustomization.getInstance(), BBConfiguration.getInstance());
        }
        asc.load(config);

        // If the config has empty values, we revert to default ones
        uri = asc.getUri();
        if (StringUtil.isNullOrEmpty(uri)) {
            uri = customization.getDefaultSourceUri(appSource.getId());
            asc.setUri(uri);
        }
        appSource.setConfig(asc);
        config.setRemoteUri(uri);

        // Re-sets the max number of items in a single SyncML message during
        // slow syncs
        int maxItemsPerMessage = BBCustomization.getInstance().getMaxItemsPerMessageInSlowSync();
        Log.info(TAG_LOG, "Setting max items in slow sync to: " + maxItemsPerMessage + " for source " + name);
        config.setMaxItemsPerMessageInSlowSync(maxItemsPerMessage);
        config.setEncoding(base64 ? SyncSource.ENCODING_B64 : SyncSource.ENCODING_NONE);

        return new ConfigPair(asc, config);
    }

    private SourceConfig initMediaSourceConfig(MediaAppSyncSourceConfig asc, MediaAppSyncSource appSource,
                                               String name, String type,
                                               String uri, boolean base64, String directory)
    {
        Log.info(TAG_LOG, "Initialize source config for [" + name + "]");
        SourceConfig config = new SourceConfig(name, type, uri);
        
        asc.load(config);

        // If the config has empty values, we revert to default ones
        uri = asc.getUri();
        if (StringUtil.isNullOrEmpty(uri)) {
            uri = customization.getDefaultSourceUri(appSource.getId());
            asc.setUri(uri);
        }
        appSource.setConfig(asc);
        config.setRemoteUri(uri);

        config.setEncoding(base64 ? SyncSource.ENCODING_B64 : SyncSource.ENCODING_NONE);

        return config;
    }


    private class ConfigPair {
        public BBAppSyncSourceConfig appSyncSourceConfig;
        public SourceConfig        sourceConfig;

        public ConfigPair(BBAppSyncSourceConfig asc, SourceConfig sc) {
            appSyncSourceConfig = asc;
            sourceConfig        = sc;
        }
    }
}

