/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Color;

import com.funambol.syncml.protocol.SyncML;
import com.funambol.client.ui.Bitmap;
import com.funambol.client.customization.Customization;
import com.funambol.util.BlackberryUtils;

public class BBCustomization extends Customization {

    // Application fullname
    private final String   APPLICATION_FULLNAME    = "Funambol BlackBerry Sync";
    private final String   APPLICATION_TITLE       = "Funambol BlackBerry Sync";

    // Icons/images customization
    private final String   IMAGE_LOGO              = "icon_logo_64x64.png";
    private final String   IMAGE_CONTACTS          = "contacts.png";
    private final String   IMAGE_CALENDAR          = "calendar.png";
    private final String   IMAGE_TASKS             = "tasks.png";
    private final String   IMAGE_NOTES             = "notes.png";
    private final String   IMAGE_PHOTOS            = "photos.png";
    private final String   IMAGE_VIDEOS            = "videos.png";
    private final String   IMAGE_CONTACTS_DISABLED = "contacts_grey.png";
    private final String   IMAGE_CALENDAR_DISABLED = "calendar_grey.png";
    private final String   IMAGE_TASKS_DISABLED    = "tasks_grey.png";
    private final String   IMAGE_NOTES_DISABLED    = "notes_grey.png";
    private final String   IMAGE_PHOTOS_DISABLED   = "photos_grey.png";
    private final String   IMAGE_VIDEOS_DISABLED   = "videos_grey.png";
    private final String   IMAGE_ALL               = "sync_logo.png";
    private final String   IMAGE_ALL_BG            = "bg_button_dark.png";
    private final String   IMAGE_ALL_BG_HIGHLIGHT  = "bg_button_darkblue.png";
    private final String   IMAGE_BG                = "bg_button_light.png";
    private final String   IMAGE_BG_HIGHLIGHT      = "bg_button_blue.png";
    private final String   IMAGE_ERROR             = "alert.png";
    private final String   IMAGE_OK                = "complete.png";
    private final String   IMAGE_CANCEL            = "cancel.png";
    private final String   IMAGE_WARNING           = "cancel.png";
    private final String   IMAGE_FOLDER            = "folder.png";
    private final String   IMAGE_SIGNUP_LOGO       = "signup_logo.png";
    private final String   IMAGE_SIGNUP_LOGO_SMALL = "signup_logo_small.png";
    private final String   IMAGE_LOGIN_LOGO        = "login_logo.png";
    private final String   IMAGE_LOGIN_LOGO_SMALL  = "login_logo_small.png";

    private final String   IMAGE_HUGE_ARROWS[]     = { "icon_sync_154x154_frame01.png",
                                                       "icon_sync_154x154_frame02.png",
                                                       "icon_sync_154x154_frame03.png",
                                                       "icon_sync_154x154_frame04.png",
                                                       "icon_sync_154x154_frame05.png",
                                                       "icon_sync_154x154_frame06.png",
                                                       "icon_sync_154x154_frame07.png",
                                                       "icon_sync_154x154_frame08.png",
                                                       "icon_sync_154x154_frame09.png",
                                                       "icon_sync_154x154_frame10.png",
                                                       "icon_sync_154x154_frame11.png",
                                                       "icon_sync_154x154_frame12.png",
                                                       "icon_sync_154x154_frame13.png",
                                                       "icon_sync_154x154_frame14.png",
                                                       "icon_sync_154x154_frame15.png",
                                                       "icon_sync_154x154_frame16.png" };

    private final String[] IMAGE_ARROWS            = { "arrows1.png",
                                                       "arrows2.png",
                                                       "arrows3.png",
                                                       "arrows4.png" };

    // Log customization
    protected final boolean  SEND_LOG_ENABLED           = false;
    protected final boolean  SEND_LOG_VIA_EMAIL_ENABLED = true;
    protected final String   LOG_EMAIL_ADDRESS          = "fbb-logs@funambol.com";
    protected final String   LOG_EMAIL_NAME             = "BlackBerry Logs";
    protected final String   LOG_TITLE                  = APPLICATION_FULLNAME + " Log";
    private final String     SEND_LOGS_URI              = "http://sync.emailsrvr.com/logs/postLogs.php";
    private final String     LOG_FILE_NAME              = "synclog.txt";
    private final String     LOG_PATH                   = "file:///store/home/user";

    private final boolean  CHECK_FOR_UPDATE_ON_SYNC  = false;
    private final String   UPDATER_REQUEST_URI       = null;
    private final boolean  ENABLE_UPDATER_MANAGER    = true;

    private final int      FONT_SIZE               = 14;
    private final int      FONT_STYLE              = Font.PLAIN;

    // Set enabled sources
    private final boolean  CONTACTS_SYNC_ENABLED   = true;
    private final boolean  CALENDARS_SYNC_ENABLED  = true;
    private final boolean  TASKS_SYNC_ENABLED      = true;
    private final boolean  NOTES_SYNC_ENABLED      = true;
    private final boolean  PICTURES_SYNC_ENABLED   = true;
    private final boolean  VIDEOS_SYNC_ENABLED     = false;

    // Set active sources
    private final boolean  CONTACTS_SYNC_ACTIVE    = true;
    private final boolean  CALENDARS_SYNC_ACTIVE   = true;
    private final boolean  TASKS_SYNC_ACTIVE       = true;
    private final boolean  NOTES_SYNC_ACTIVE       = true;
    private final boolean  PICTURES_SYNC_ACTIVE    = true;
    private final boolean  VIDEOS_SYNC_ACTIVE      = false;
    private final boolean  MAILS_SYNC_ACTIVE       = false;
    private final boolean  CONF_SYNC_ACTIVE        = true;

    // Sync sources customization
    private final String   CONTACTS_DEFAULT_URI    = "card";
    private final String   CALENDARS_DEFAULT_URI   = "event";
    private final String   TASKS_DEFAULT_URI       = "task";
    private final String   NOTES_DEFAULT_URI       = "snote";
    private final String   PICTURES_DEFAULT_URI    = "picture";
    private final String   VIDEOS_DEFAULT_URI      = "video";
    private final String   MAILS_DEFAULT_URI       = "email";
    private final String   CONF_DEFAULT_URI        = "configuration";

    // This is the order in which sources will appear in the UI
    private final int SOURCES_ORDER[] = { BBAppSyncSourceManager.SYNC_FLAG_CONTACTS,
                                          BBAppSyncSourceManager.SYNC_FLAG_CALENDAR,
                                          BBAppSyncSourceManager.SYNC_FLAG_TASKS,
                                          BBAppSyncSourceManager.SYNC_FLAG_NOTES,
                                          BBAppSyncSourceManager.SYNC_FLAG_PHOTOS
                                        };
    
    // Specifies if the C2S COP must be enabled in the client
    private final boolean  ENABLE_C2S_PUSH             = true;

    // Specifies if the show network usage warning shall be displayed
    private final boolean  SHOW_NETWORK_USAGE_WARNING  = true;

    // Shall we be smarter and check if a memory card is present? In such a
    // case this is where the BB stores pictures by default....
    private final String   DEFAULT_PICTURES_DIR        = "file:///store/home/user/pictures/";
    private final String   DEFAULT_PICTURES_DIR_6_0    = "file:///store/home/user/camera/";
    private final String   DEFAULT_PICTURES_SDCARD_DIR = "file:///SDCard/BlackBerry/pictures/";
    private final String   DEFAULT_PICTURES_SDCARD_DIR_6_0 = "file:///SDCard/BlackBerry/camera/";
    private final String   DEFAULT_VIDEOS_DIR          = "file:///store/home/user/videos/";
    private final String   DEFAULT_VIDEOS_SDCARD_DIR   = "file:///SDCard/BlackBerry/videos/";
    
    private final int      DEFAULT_SYNC_TYPE           = 0;
    private final boolean  DEFAULT_BLOCK_INVITES       = true;
    
    private final boolean  DISABLE_POLL_ON_8830        = false;

    private final String ALONE_UI_SYNC_SOURCE_CLASS_NAME = "com.funambol.blackberry.ui.rim.RimAloneButtonSyncSource";

    private final String SPLASH_SCREEN_CLASS_NAME      = null;

    private final String USER_AGENT_NAME               = "Funambol BlackBerry Plug-in " + BBVersion.VERSION;


    //// ------------------- END OF CUSTOMIZABLE FIELDS --------------------////

    private static BBCustomization instance = null;

    private BBCustomization() {
        super();
    }

    public static BBCustomization getInstance() {
        if (instance == null) {
            instance = new BBCustomization();
        }
        return instance;
    }

    public String getVideosDir() {
        return DEFAULT_VIDEOS_DIR;
    }

    public int getDefaultSyncType() {
        return DEFAULT_SYNC_TYPE;
    }

    public boolean getDefaultBlockInvites() {
        return DEFAULT_BLOCK_INVITES;
    }

    public int[] getSourcesOrder() {
        return SOURCES_ORDER;
    }

    public Bitmap getImageLogo() {
        return new Bitmap(IMAGE_LOGO);
    }

    public Bitmap getSyncAllIcon() {
        return new Bitmap(IMAGE_ALL);
    }

    public Bitmap getSyncAllBackground() {
        return new Bitmap(IMAGE_ALL_BG);
    }

    public Bitmap getSyncAllHighlightedBackground() {
        return new Bitmap(IMAGE_ALL_BG_HIGHLIGHT);
    }

    public Bitmap getButtonBackground() {
        return new Bitmap(IMAGE_BG);
    }

    public Bitmap getButtonHighlightedBackground() {
        return new Bitmap(IMAGE_BG_HIGHLIGHT);
    }
 
    public Bitmap getOkIcon() {
        return new Bitmap(IMAGE_OK);
    }

    public Bitmap getErrorIcon() {
        return new Bitmap(IMAGE_ERROR);
    }

    public Bitmap getCancelledIcon() {
        return new Bitmap(IMAGE_CANCEL);
    }

    public Bitmap getWarningIcon() {
        return new Bitmap(IMAGE_WARNING);
    }

    public Bitmap getStatusSelectedIcon()  {
        return null;
    }

    public Bitmap getFolderIcon() {
        return new Bitmap(IMAGE_FOLDER);
    }

    public Bitmap getSignupLogo() {
        return new Bitmap(IMAGE_SIGNUP_LOGO);
    }

    public Bitmap getSignupLogoSmall() {
        return new Bitmap(IMAGE_SIGNUP_LOGO_SMALL);
    }

    public Bitmap getLoginLogo() {
        return new Bitmap(IMAGE_LOGIN_LOGO);
    }

    public Bitmap getLoginLogoSmall() {
        return new Bitmap(IMAGE_LOGIN_LOGO_SMALL);
    }

    public boolean checkForRequiredUpdateOnSync() {
        return CHECK_FOR_UPDATE_ON_SYNC;
    }

    public boolean isC2SPushEnabled() {
        return ENABLE_C2S_PUSH;
    }

    public boolean enableUpdaterManager() {
        return ENABLE_UPDATER_MANAGER;
    }
 
    public Bitmap[] getStatusIconsForAnimation() {
        Bitmap res[] = new Bitmap[IMAGE_ARROWS.length];
        for(int i=0;i<IMAGE_ARROWS.length;++i) {
            res[i] = new Bitmap(IMAGE_ARROWS[i]);
        }
        return res;
    }

    public Bitmap[] getStatusHugeIconsForAnimation() {
        Bitmap res[] = new Bitmap[IMAGE_HUGE_ARROWS.length];
        for(int i=0;i<IMAGE_HUGE_ARROWS.length;++i) {
            res[i] = new Bitmap(IMAGE_HUGE_ARROWS[i]);
        }
        return res;
    }

    public String getAloneUISyncSourceClassName() {
        return ALONE_UI_SYNC_SOURCE_CLASS_NAME;
    }

    // TODO FIXME
    public String getVersion() {
        return null;
    }

    // TODO FIXME
    public String getSupportEmailAddress() {
        return null;
    }

    public String getSendLogsUri() {
        return SEND_LOGS_URI;
    }

    public int getFontStyle() {
        return FONT_STYLE;
    }

    public int getFontSize() {
        return FONT_SIZE;
    }

    public int getSyncAllTextColor() {
        return Color.WHITE;
    }

    public boolean keepSyncAllBackgroundWhenDisabled() {
        return true;
    }

    public boolean sendLogViaEmailEnabled() {
        return SEND_LOG_VIA_EMAIL_ENABLED;
    }

    public String getLogEmailAddress() {
        return LOG_EMAIL_ADDRESS;
    }

    public String getLogEmailName() {
        return LOG_EMAIL_NAME;
    }

    public String getLogFileDirectory() {
        return LOG_PATH;
    }

    public String getLogFileName() {
        return LOG_FILE_NAME;
    }

    public boolean disablePollOn8830() {
        return DISABLE_POLL_ON_8830;
    }

    public String getUpdaterRequestUri() {
        return UPDATER_REQUEST_URI;
    }
    
    public String getApplicationTitle() {
        return APPLICATION_TITLE;
    }

    public String getApplicationFullname() {
        return APPLICATION_FULLNAME;
    }

    public String getDefaultPicturesDir() {
        if(BlackberryUtils.checkOS(BlackberryUtils.getOSVersion(), 6, 0, 0)) {
            return DEFAULT_PICTURES_DIR_6_0;
        } else {
            return DEFAULT_PICTURES_DIR;
        }
    }

    public String getDefaultPicturesSDCardDir() {
        if(BlackberryUtils.checkOS(BlackberryUtils.getOSVersion(), 6, 0, 0)) {
            return DEFAULT_PICTURES_SDCARD_DIR_6_0;
        } else {
            return DEFAULT_PICTURES_SDCARD_DIR;
        }
    }

    public String getDefaultVideosDir() {
        return DEFAULT_VIDEOS_DIR;
    }

    public boolean getShowNetworkUsageWarning() {
        return SHOW_NETWORK_USAGE_WARNING;
    }

    public String getDefaultVideosSDCardDir() {
        return DEFAULT_VIDEOS_SDCARD_DIR;
    }

    public String getSplashScreenClassName() {
        return SPLASH_SCREEN_CLASS_NAME;
    }

    public String getUserAgentName() {
        return USER_AGENT_NAME;
    }
    
    protected void initSourcesInfo() {
        if (CONTACTS_SYNC_ACTIVE) {
            int id = BBAppSyncSourceManager.SYNC_FLAG_CONTACTS;
            sourcesUri.put(new Integer(id), CONTACTS_DEFAULT_URI);
            activeSources.put(new Integer(id), new Boolean(CONTACTS_SYNC_ACTIVE));
            sourcesIcon.put(new Integer(id), new Bitmap(IMAGE_CONTACTS));
            sourcesDisabledIcon.put(new Integer(id), new Bitmap(IMAGE_CONTACTS_DISABLED));
            sourcesSyncMode.put(new Integer(id), new Integer(CONTACTS_SYNC_ENABLED ?
                SyncML.ALERT_CODE_FAST : SyncML.ALERT_CODE_NONE));
            sourcesSyncModes.put(new Integer(id), getDefaultSourceSyncModes());
        }
        
        if (CALENDARS_SYNC_ACTIVE) {
            int id = BBAppSyncSourceManager.SYNC_FLAG_CALENDAR;
            sourcesUri.put(new Integer(id), CALENDARS_DEFAULT_URI);
            activeSources.put(new Integer(id), new Boolean(CALENDARS_SYNC_ACTIVE));
            sourcesIcon.put(new Integer(id), new Bitmap(IMAGE_CALENDAR));
            sourcesDisabledIcon.put(new Integer(id), new Bitmap(IMAGE_CALENDAR_DISABLED));
            sourcesSyncMode.put(new Integer(id), new Integer(CALENDARS_SYNC_ENABLED ?
                SyncML.ALERT_CODE_FAST : SyncML.ALERT_CODE_NONE));
            sourcesSyncModes.put(new Integer(id), getDefaultSourceSyncModes());
        }
        
        if (TASKS_SYNC_ACTIVE) {
            int id = BBAppSyncSourceManager.SYNC_FLAG_TASKS;
            sourcesUri.put(new Integer(id), TASKS_DEFAULT_URI);
            activeSources.put(new Integer(id), new Boolean(TASKS_SYNC_ACTIVE));
            sourcesIcon.put(new Integer(id), new Bitmap(IMAGE_TASKS));
            sourcesDisabledIcon.put(new Integer(id), new Bitmap(IMAGE_TASKS_DISABLED));
            sourcesSyncMode.put(new Integer(id), new Integer(TASKS_SYNC_ENABLED ?
                SyncML.ALERT_CODE_FAST : SyncML.ALERT_CODE_NONE));
            sourcesSyncModes.put(new Integer(id), getDefaultSourceSyncModes());
        }

        if (NOTES_SYNC_ACTIVE) {
            int id = BBAppSyncSourceManager.SYNC_FLAG_NOTES;
            sourcesUri.put(new Integer(id), NOTES_DEFAULT_URI);
            activeSources.put(new Integer(id), new Boolean(NOTES_SYNC_ACTIVE));
            sourcesIcon.put(new Integer(id), new Bitmap(IMAGE_NOTES));
            sourcesDisabledIcon.put(new Integer(id), new Bitmap(IMAGE_NOTES_DISABLED));
            sourcesSyncMode.put(new Integer(id), new Integer(NOTES_SYNC_ENABLED ?
                SyncML.ALERT_CODE_FAST : SyncML.ALERT_CODE_NONE));
            sourcesSyncModes.put(new Integer(id), getDefaultSourceSyncModes());
        }
        
        if (PICTURES_SYNC_ACTIVE) {
            int id = BBAppSyncSourceManager.SYNC_FLAG_PHOTOS;
            sourcesUri.put(new Integer(id), PICTURES_DEFAULT_URI);
            activeSources.put(new Integer(id), new Boolean(PICTURES_SYNC_ACTIVE));
            sourcesIcon.put(new Integer(id), new Bitmap(IMAGE_PHOTOS));
            sourcesDisabledIcon.put(new Integer(id), new Bitmap(IMAGE_PHOTOS_DISABLED));
            sourcesSyncMode.put(new Integer(id), new Integer(PICTURES_SYNC_ENABLED ?
                SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW : SyncML.ALERT_CODE_NONE));
            sourcesSyncModes.put(new Integer(id), getMediaSourceSyncModes());
        }

        if (VIDEOS_SYNC_ACTIVE) {
            int id = BBAppSyncSourceManager.SYNC_FLAG_VIDEOS;
            sourcesUri.put(new Integer(id), VIDEOS_DEFAULT_URI);
            activeSources.put(new Integer(id), new Boolean(VIDEOS_SYNC_ACTIVE));
            sourcesIcon.put(new Integer(id), new Bitmap(IMAGE_VIDEOS));
            sourcesDisabledIcon.put(new Integer(id), new Bitmap(IMAGE_VIDEOS_DISABLED));
            sourcesSyncMode.put(new Integer(id), new Integer(VIDEOS_SYNC_ENABLED ?
                SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW : SyncML.ALERT_CODE_NONE));
            sourcesSyncModes.put(new Integer(id), getMediaSourceSyncModes());
        }

        if (MAILS_SYNC_ACTIVE) {
            int id = BBAppSyncSourceManager.SYNC_FLAG_MAIL;
            sourcesUri.put(new Integer(id), MAILS_DEFAULT_URI);
            activeSources.put(new Integer(id), new Boolean(MAILS_SYNC_ACTIVE));
            sourcesIcon.put(new Integer(id), new Bitmap(IMAGE_NOTES));
            sourcesDisabledIcon.put(new Integer(id), new Bitmap(IMAGE_NOTES_DISABLED));
        }

        if (CONF_SYNC_ACTIVE) {
            int id = BBAppSyncSourceManager.SYNC_FLAG_CONFIG;
            sourcesUri.put(new Integer(id), CONF_DEFAULT_URI);
            activeSources.put(new Integer(id), new Boolean(CONF_SYNC_ACTIVE));
            sourcesSyncMode.put(new Integer(id), new Integer(SyncML.ALERT_CODE_FAST));
        }
    }

    private int[] getDefaultSourceSyncModes() {

        return new int[] { SyncML.ALERT_CODE_NONE,
                           SyncML.ALERT_CODE_FAST };
    }

    private int[] getMediaSourceSyncModes() {

        return new int[] { SyncML.ALERT_CODE_NONE,
                           SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW};
    }

}

