/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import com.funambol.blackberry.BBConfiguration;
import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.blackberry.ui.rim.RimProgressDialog;
import com.funambol.client.configuration.Configuration;
import com.funambol.client.controller.AccountScreenController;
import com.funambol.client.controller.SignupScreenController;
import com.funambol.client.customization.Customization;
import com.funambol.client.localization.Localization;
import com.funambol.client.source.AppSyncSourceManager;
import com.funambol.client.ui.SignupScreen;
import com.funambol.platform.DeviceInfo;
import com.funambol.platform.DeviceInfoInterface;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;


public class BBSignupScreenController extends SignupScreenController {

    private static final String TAG_LOG = "BBSignupScreenController";

    private Controller bbController = null;
    private DisplayManager displayManager = null;
    private RimProgressDialog progressDialog = null;

    private boolean networkUsageWarningDisplayed = false;

    public BBSignupScreenController(Controller controller, Customization customization,
            Configuration configuration, Localization localization,
            AppSyncSourceManager appSyncSourceManager, SignupScreen signupScreen) {
        super(ProxyController.getInstance(), customization, configuration,
                localization, appSyncSourceManager, signupScreen);
        bbController = controller;
        displayManager = bbController.getDisplayManager();
    }

    protected DeviceInfoInterface getDeviceInfo() {
        return new DeviceInfo();
    }

    protected void showProgressDialog(String label) {
        if(progressDialog != null) {
            hideProgressDialog();
        }
        progressDialog = new RimProgressDialog();
        progressDialog.setLabel(label);
        if (progressDialog != null) {
            DisplayManager dm = bbController.getDisplayManager();
            try {
                dm.showScreen(progressDialog);
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot display progress dialog", e);
            }
        }
    }

    protected void hideProgressDialog() {
        if (progressDialog != null) {
            try {
                displayManager.hideScreen(progressDialog);
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot close progress dialog", e);
            } finally {
                progressDialog = null;
            }
        }
    }

    protected boolean isSyncInProgress() {
        HomeScreenController homeScreenController = bbController.getHomeScreenController();
        return homeScreenController.isSynchronizing();
    }

    protected void showMessage(String msg) {
        bbController.getDialogController().showMessage(msg);
    }

    protected void showMessage(String msg, int delay) {
        bbController.getDialogController().showMessage(msg, delay);
    }

    protected void showSyncInProgressMessage() {
        // If the home screen is not displayed, we cannot show any warning and
        // just ignore this event
        if (screen != null) {
            DisplayManager dm = bbController.getDisplayManager();
            String msg = localization.getLanguage("message_sync_running_wait");
            dm.showMessage(msg);
        }
    }

    protected void userAuthenticated() {
        Log.info(TAG_LOG, "User authenticated");
        
        bbController.getScreenDisplayController().hideSignupScreen();
        bbController.getScreenDisplayController().showHomeScreen();

        updateLoginScreen();

        super.userAuthenticated();
    }

    public void switchToLoginScreen() {
        updateLoginScreen();
        bbController.getScreenDisplayController().hideSignupScreen();
        bbController.getScreenDisplayController().showLoginScreen(true);
    }

    private void updateLoginScreen() {
        AccountScreenController loginScreenController =
               bbController.getLoginScreenController();
        loginScreenController.initScreen(screen.getSyncUrl(),
                                         screen.getUsername(),
                                         screen.getPassword());
    }

    public void showNetworkUsageWarningIfRequired() {
        Log.info(TAG_LOG, "Checking if we must show the network warning");
        // Do we need to show the network usage prompt?
        if (((BBCustomization)customization).getShowNetworkUsageWarning()
                && !networkUsageWarningDisplayed) {
            if (!((BBConfiguration)configuration).getNetwokUsageWarningAccepted()) {
                NetworkUsageWarningThread t = new NetworkUsageWarningThread();
                t.start();
            }
        }
    }

    private class NetworkUsageWarningThread extends Thread {
        public void run() {
            networkUsageWarningDisplayed = true;

            Log.info(TAG_LOG, "Showing network usage warning");
            DialogController dc = bbController.getDialogController();
            int values[] = {0, 1};
            String msg = localization.getLanguage("dialog_network_usage_warning");
            String options[] = { localization.getLanguage("dialog_ok"),
                localization.getLanguage("dialog_exit") };
            int res = dc.promptSelection(msg, options, values, 0, true, DisplayManager.NO_LIMIT);
            Log.info(TAG_LOG, "User answer: " + res);
            if (res == 0) {
                ((BBConfiguration)configuration).setNetwokUsageWarningAccepted(true);
                configuration.save();
            } else {
                System.exit(0);
            }
        }
    }
}
