/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;

import com.funambol.client.configuration.Configuration;
import com.funambol.client.customization.Customization;
import com.funambol.client.source.AppSyncSourceConfig;
import com.funambol.client.source.AppSyncSource;
import com.funambol.client.source.AppSyncSourceManager;
import com.funambol.syncml.spds.SyncListener;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.protocol.SyncML;

import com.funambol.util.Log;

public class BBAppSyncSourceConfig extends AppSyncSourceConfig {

    private static final String TAG_LOG = "BBAppSyncSourceConfig";

    public BBAppSyncSourceConfig(AppSyncSource appSource, Customization customization, Configuration configuration) {
        super(appSource, customization, configuration);
    }

    public void save() {
        super.save();
        // For backward compatibility on BB the save perfoms the commit
        // automatically
        commit();
    }

    protected void migrateConfig(String from, String to, SourceConfig config) {

        int sourceId = appSource.getId();

        // the version in the app sync source config was introduced in v9. In
        // this case the from is null because not available in previous version
        if (from == null) {
            int oldId = sourceId;
            // Two sources swapped their ids in this version
            if (sourceId == 16) {
                oldId = 32;
            } else if (sourceId == 32) {
                oldId = 16;
            }

            // Migrates from v8.7 to v9.0
            Log.trace(TAG_LOG, "Migrating from v87 to v90");

            // Migrate the enabled key
            StringBuffer oldKey = new StringBuffer();
            oldKey.append("SYNC_SOURCE").append("_").append(oldId);
            enabled = configuration.loadBooleanKey(oldKey.toString(), true);

            // Migrate the active key
            oldKey = new StringBuffer();
            oldKey.append("SYNC_SOURCE_ACTIVE").append("_").append(oldId);
            active = configuration.loadBooleanKey(oldKey.toString(), true);

            // Migrate the source full flag
            oldKey = new StringBuffer();
            oldKey.append(CONF_KEY_SOURCE_FULL).append("_").append(oldId);
            deviceFullShown = configuration.loadBooleanKey(oldKey.toString(), false);

            // Migrate the upload via HTTP flag
            oldKey = new StringBuffer();
            oldKey.append("UPLOAD_VIA_HTTP").append("_").append(oldId);
            uploadContentViaHttp = configuration.loadBooleanKey(oldKey.toString(), false);

            // Migrate the status
            oldKey = new StringBuffer();
            oldKey.append("LAST_SYNC_STATUS").append("_").append(oldId);
            lastSyncStatus = configuration.loadIntKey(oldKey.toString(), SyncListener.SUCCESS);
            
            // Migrate the source uri
            oldKey = new StringBuffer();
            oldKey.append("SYNC_SOURCE_URI").append("_").append(oldId);
            uri = configuration.loadStringKey(oldKey.toString(), uri);

            // Migrate the synced flag
            oldKey = new StringBuffer();
            oldKey.append("SYNC_SOURCE_SYNCED").append("_").append(oldId);
            sourceSynced = configuration.loadBooleanKey(oldKey.toString(), false);

            // Now migrate the source config (these are the old keys)
            if (appSource.getId() == AppSyncSourceManager.CONTACTS_ID) {
                migrateSourceConfig("SYNC_CONTACT_CONFIG", config);
            }

            if (appSource.getId() == AppSyncSourceManager.EVENTS_ID) {
                migrateSourceConfig("SYNC_CAL_CONFIG", config);
            }

            if (appSource.getId() == AppSyncSourceManager.TASKS_ID) {
                migrateSourceConfig("SYNC_TASK_CONFIG", config);
            }

            if (appSource.getId() == AppSyncSourceManager.NOTES_ID) {
                migrateSourceConfig("SYNC_NOTE_CONFIG", config);
            }

            if (appSource.getId() == AppSyncSourceManager.PICTURES_ID) {
                migrateSourceConfig("SYNC_PHOTO_CONFIG", config);
            }

            // The sync type in v87 was global, now it is per source. We migrate
            // only pim sources, because media one were not affected by the
            // global direction setting
            if (!appSource.getIsMedia()) {
                int oldSyncType = configuration.loadIntKey("SYNC_TYPE", 0);
                // Convert from the old to the new representation  (if none of the
                // below, we keep the default from the customization)
                if (oldSyncType == 0) {
                    syncType = SyncML.ALERT_CODE_FAST;
                    // TWO WAYS
                } else if (oldSyncType == 1) {
                    // ONE WAY TO SERVER
                    syncType = SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT;
                } else if (oldSyncType == 2) {
                    // ONE WAY TO CLIENT
                    syncType = SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER;
                }
            }
        }

        // Now migrate the basic config
        super.migrateConfig(from, to, config);
    }

    private void migrateSourceConfig(String oldKey, SourceConfig config) {
        if (config != null) {
            byte configBytes[] = configuration.loadByteArrayKey(oldKey, null);
            if (configBytes != null) {
                try {
                    Log.info(TAG_LOG, "Migrating " + appSource.getName() + " source config");
                    DataInputStream temp = new DataInputStream(new ByteArrayInputStream(configBytes));
                    config.deserialize(temp);
                    temp.close();
                } catch (IOException ioe) {
                    Log.error(TAG_LOG, "Cannot migrate config for source " + appSource.getName(), ioe);
                }
            }
        }
    }
}

