/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source.media;

import java.io.IOException;

import com.funambol.blackberry.source.BBAppSyncSourceConfig;

import com.funambol.syncml.spds.SourceConfig;
import com.funambol.client.source.AppSyncSourceConfig;
import com.funambol.client.source.AppSyncSourceManager;
import com.funambol.client.configuration.Configuration;
import com.funambol.client.customization.Customization;
import com.funambol.util.Log;

/**
 * This class represents the configuration of an AppSyncSource. In particular it
 * holds all the values that are kept across application resets. These values
 * are loaded/saved using a Configuration object.
 */
public class MediaAppSyncSourceConfig extends BBAppSyncSourceConfig {

    private static final String TAG_LOG = "MediaAppSyncSourceConfig";

    private static final String CONF_KEY_DIRECTORY              = "SYNC_SOURCE_DIRECTORY";
    private static final String CONF_KEY_SYNC_OLD               = "SYNC_SOURCE_OLD";

    private String directory;
    private boolean syncOld = false;

    public MediaAppSyncSourceConfig(MediaAppSyncSource appSource, Customization customization,
                                    Configuration configuration, String directory)
    {
        super(appSource, customization, configuration);
        this.directory = directory;
    }

    public void save() {
        Log.trace(TAG_LOG, "Saving MediaAppSyncSourceConfig");
        int sourceId = appSource.getId();
        StringBuffer key = new StringBuffer();
        key.append(CONF_KEY_DIRECTORY).append("-").append(sourceId);
        configuration.saveStringKey(key.toString(), directory);

        key = new StringBuffer();
        key.append(CONF_KEY_SYNC_OLD).append("-").append(sourceId);
        configuration.saveBooleanKey(key.toString(), syncOld);
        super.save();
    }

    public void load(SourceConfig sc) {
        Log.trace(TAG_LOG, "Loading MediaAppSyncSourceConfig");
        int sourceId = appSource.getId();
        super.load(sc);
        StringBuffer key = new StringBuffer();
        key.append(CONF_KEY_DIRECTORY).append("-").append(sourceId);
        directory = configuration.loadStringKey(key.toString(), directory);

        key = new StringBuffer();
        key.append(CONF_KEY_SYNC_OLD).append("-").append(sourceId);
        syncOld = configuration.loadBooleanKey(key.toString(), syncOld);
    }

    public String getDirectory() {
        return directory;
    }

    public void setDirectory(String directory) {
        this.directory = directory;
    }

    public boolean getIncludeOld() {
        return syncOld;
    }

    public void setIncludeOld(boolean includeOld) {
        this.syncOld = includeOld;
    }

    protected void migrateConfig(String from, String to, SourceConfig config) {
        // This upgrade code is only for picture source
        if (from == null) {
            // This is the upgrade from v87 to v9
            if (appSource.getId() == AppSyncSourceManager.PICTURES_ID) {
                Log.info(TAG_LOG, "Migrating pictures config");
                if (from == null) {
                    // Migrate the sync old
                    syncOld = configuration.loadBooleanKey("PICTURES_SYNC_OLD", false);
                    // Migrate the pictures dir
                    String  picsDir     = configuration.loadStringKey("PICTURES_DIR", null);
                    Log.trace(TAG_LOG, "picsDir = " + picsDir);
                    if (picsDir != null) {
                        directory = picsDir;
                    }
                }
            }
        }
        super.migrateConfig(from, to, config);
    }
}
 
