/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source.media.picture;

import com.funambol.blackberry.BBConfiguration;
import com.funambol.blackberry.source.ExtendedSyncSource;
import com.funambol.blackberry.source.media.picture.PictureSyncSource;
import com.funambol.blackberry.source.media.MediaAppSyncSourceConfig;
import com.funambol.blackberry.source.media.picture.TwoPhasesPictureSyncSource;
import com.funambol.blackberry.source.media.MediaAppSyncSource;
import com.funambol.blackberry.controller.UISyncSourceController;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.HomeScreenController;
import com.funambol.client.source.AppSyncSource;
import com.funambol.client.source.AppSyncSourceConfig;
import com.funambol.client.configuration.Configuration;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncSource;

import com.funambol.util.Log;

public class PictureAppSyncSource extends MediaAppSyncSource {

    private static final String TAG_LOG = "PictureAppSyncSource";

    protected SyncSource syncMLSource;
    protected TwoPhasesPictureSyncSource twoPhasesSyncSource;
    private Configuration configuration = BBConfiguration.getInstance();

    public PictureAppSyncSource(String name, SyncSource source) {
        super(name, source);
    }

    public PictureAppSyncSource(String name) {
        super(name);
    }

    public void reapplyConfiguration() {

        super.reapplyConfiguration();
        AppSyncSourceConfig appConfig = getConfig();
        // Change the upload mode if required
        if (appConfig.getUploadContentViaHttp() && twoPhasesSyncSource != null) {
            Log.info(TAG_LOG, "Switching to two phases sync source");
            setSyncSource(twoPhasesSyncSource);
        } else {
            Log.info(TAG_LOG, "Switching to pure syncml sync source");
            if (syncMLSource != null) {
                setSyncSource(syncMLSource);
            } else {
                Log.error(TAG_LOG, "Invalid syncml source, source will be disabled");
            }
        }
    }

    public void setSyncMLSource(SyncSource source) {
        syncMLSource = source;
    }

    public void setTwoPhasesSyncSource(TwoPhasesPictureSyncSource source) {
        twoPhasesSyncSource = source;
    }

    public void setClientUISyncSourceController(UISyncSourceController controller) {
        super.setClientUISyncSourceController(controller);
        Controller globalController = controller.getController();
        twoPhasesSyncSource.setController(globalController.getHomeScreenController());
    }

}


