/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source.pim.calendar;

import com.funambol.blackberry.source.BBAppSyncSourceConfig;

import com.funambol.syncml.spds.SourceConfig;
import com.funambol.client.configuration.Configuration;
import com.funambol.client.customization.Customization;
import com.funambol.client.source.AppSyncSource;

/**
 * This class represents the configuration of an AppSyncSource. In particular it
 * holds all the values that are kept across application resets. These values
 * are loaded/saved using a Configuration object.
 */
public class CalendarAppSyncSourceConfig extends BBAppSyncSourceConfig {

    private static final String TAG_LOG = "CalendarAppSyncSourceConfig";

    private static final String CONF_KEY_BLOCK_INCOMING_INVITES = "BLOCK_INVITES";
    private static final String CONF_KEY_SYNC_RANGE_PAST        = "RANGE_PAST";
    private static final String CONF_KEY_SYNC_RANGE_FUTURE      = "RANGE_FUTURE";

    private boolean blockIncomingInvites = false;
    private int     rangePast;
    private int     rangeFuture;

    public CalendarAppSyncSourceConfig(AppSyncSource appSource, 
            Customization customization, Configuration configuration) {
        super(appSource, customization, configuration);
    }

    public void save() {
        configuration.saveBooleanKey(CONF_KEY_BLOCK_INCOMING_INVITES, blockIncomingInvites);
        configuration.saveIntKey(CONF_KEY_SYNC_RANGE_PAST, rangePast);
        configuration.saveIntKey(CONF_KEY_SYNC_RANGE_FUTURE, rangeFuture);
        super.save();
    }

    public void load(SourceConfig sc) {
        super.load(sc);
        blockIncomingInvites = configuration.loadBooleanKey(
                CONF_KEY_BLOCK_INCOMING_INVITES, customization.getDefaultBlockInvites());
        rangePast = configuration.loadIntKey(CONF_KEY_SYNC_RANGE_PAST, 
                customization.getDefaultRangePast());
        rangeFuture = configuration.loadIntKey(CONF_KEY_SYNC_RANGE_FUTURE,
                customization.getDefaultRangeFuture());
    }

    public boolean getBlockIncomingInvites() {
        return blockIncomingInvites;
    }

    public void setBlockIncomingInvites(boolean blockIncomingInvites) {
        this.blockIncomingInvites = blockIncomingInvites;
    }

    public int getRangePast() {
        return rangePast;
    }

    public void setRangePast(int range) {
        rangePast = range;
    }

    public int getRangeFuture() {
        return rangeFuture;
    }

    public void setRangeFuture(int range) {
        rangeFuture = range;
    }
}
 
