/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source.pim.calendar;

import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.ui.rim.RimSourceBasicSettings;
import com.funambol.client.configuration.Configuration;
import com.funambol.client.customization.Customization;
import com.funambol.util.Log;

import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.NumericChoiceField;
import net.rim.device.api.ui.container.HorizontalFieldManager;

public class CalendarSourceSettings extends RimSourceBasicSettings {

    private static final String TAG_LOG = "CalendarSourceSettings";

    private LabelField blockInvitesLabel    = null;
    private CheckboxField blockInvitesField = null;
    
    private LabelField rangeLabel = null;
    private NumericChoiceField rangePastField   = null;
    private NumericChoiceField rangeFutureField = null;

    private boolean blockInvites;
    private int rangePast;
    private int rangeFuture;

    private Customization customization = null;
    
    public CalendarSourceSettings() {
        super();
        customization = BBCustomization.getInstance();
    }

    public void layout() {
        // First of all the standard settings
        super.layout();
        if (customization.isBlockInvitesFieldVisible()) {
            blockInvitesLabel = new LabelField(localization
                    .getLanguage("conf_block-invites"));
            blockInvitesLabel.setFont(getFont().derive(Font.BOLD));
            blockInvitesField = new CheckboxField("", blockInvites);
            blockInvitesField.setChecked(blockInvites);
            blockInvitesField.setChangeListener(cl);

            hfm = new HorizontalFieldManager();
            hfm.add(blockInvitesField);
            hfm.add(blockInvitesLabel);
            vfm.add(hfm);
        }

        if (customization.isRangePastVisible() || customization.isRangeFutureVisible()) {
            rangeLabel = new LabelField(localization.getLanguage("conf_range"));
            rangeLabel.setFont(getFont().derive(Font.BOLD));
            vfm.add(rangeLabel);
        }
        if (customization.isRangePastVisible()) {
            rangePastField = new NumericChoiceField(localization.getLanguage("conf_range-past")
                    + "  ", 
                    1, // begin
                    3, // end
                    1, // increment
                    rangePast - 1);
            rangePastField.setEditable(false);
            rangePastField.setFont(getFont());
            rangePastField.setChangeListener(cl);
            vfm.add(rangePastField);
        }
        if (customization.isRangeFutureVisible()) {
            rangeFutureField = new NumericChoiceField(localization.getLanguage("conf_range-future")
                    + "  ", 
                    1, // begin
                    6, // end
                    1, // increment
                    rangeFuture - 1);
            rangeFutureField.setEditable(false);
            rangeFutureField.setFont(getFont());
            rangeFutureField.setChangeListener(cl);
            vfm.add(rangeFutureField);
        }
    }

    public void loadSettings(Configuration configuration) {
        super.loadSettings(configuration);
        if (source != null) {
            Log.trace(TAG_LOG, "Loading settings for " + source.getName());
            CalendarAppSyncSourceConfig appConfig =
                    (CalendarAppSyncSourceConfig)source.getConfig();
            if (customization.isBlockInvitesFieldVisible()) {
                blockInvites = appConfig.getBlockIncomingInvites();
            }
            if (customization.isRangePastVisible()) {
                rangePast = appConfig.getRangePast();
            }
            if(customization.isRangeFutureVisible()) {
                rangeFuture = appConfig.getRangeFuture();
            }
        }
    }

    public void saveSettings(Configuration configuration) {
        super.saveSettings(configuration);
        if (source != null) {
            Log.trace(TAG_LOG, "Saving settings for " + source.getName());
            CalendarAppSyncSourceConfig appConfig =
                    (CalendarAppSyncSourceConfig)source.getConfig();
            if (customization.isBlockInvitesFieldVisible()) {
                appConfig.setBlockIncomingInvites(getBlockInvitesFromUi());
            }
            if (customization.isRangePastVisible()) {
                appConfig.setRangePast(getRangePastFromUi());
            }
            if(customization.isRangeFutureVisible()) {
                appConfig.setRangeFuture(getRangeFutureFromUi());
            }
        }
        dirty = false;
    }

    private boolean getBlockInvitesFromUi() {
        return blockInvitesField.getChecked();
    }

    private int getRangePastFromUi() {
        return rangePastField.getSelectedValue();
    }

    private int getRangeFutureFromUi() {
        return rangeFutureField.getSelectedValue();
    }
}
