/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source.pim.contact;

import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.ui.rim.RimSourceBasicSettings;
import com.funambol.blackberry.ui.rim.component.ColoredLabelField;
import com.funambol.client.configuration.Configuration;
import com.funambol.client.customization.Customization;
import com.funambol.util.Log;

import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.container.HorizontalFieldManager;

public class ContactSourceSettings extends RimSourceBasicSettings {

    private static final String TAG_LOG = "ContactSourceSettings";

    private Customization customization = null;

    private CheckboxField cdEnableField;
    private ColoredLabelField cdLabelField;

    public ContactSourceSettings() {
        super();
        customization = BBCustomization.getInstance();

        cdLabelField = new ColoredLabelField(
                localization.getLanguage("custom_config_cd"));
        cdEnableField = new CheckboxField("", false);
    }

    public void layout() {
        // First of all the standard settings
        super.layout();
        if (customization.syncCompanyDirectory()) {

            FieldChangeListener listener = new CDListener(cl, uriField);
            cdEnableField.setChangeListener(listener);

            if (!source.getConfig().getEnabled()) {
                cdEnableField.setEditable(false);
                cdLabelField.setColor(Color.GRAY);
            }
            HorizontalFieldManager hfm2 = new HorizontalFieldManager();
            hfm2.add(cdEnableField);
            hfm2.add(cdLabelField);
            vfm.add(hfm2);
        }
    }

    public void loadSettings(Configuration configuration) {
        super.loadSettings(configuration);
        if (source != null && customization.syncCompanyDirectory()) {
            Log.trace(TAG_LOG, "Loading settings for " + source.getName());
            String uri = source.getConfig().getUri();
            cdEnableField.setChecked(uri.endsWith("-shared"));
        }
    }

    private class CDListener implements FieldChangeListener {

        private ChangeListener listener;
        private BasicEditField  field;

        public CDListener(ChangeListener cl, BasicEditField field) {
            listener   = cl;
            this.field = field;
        }

        public void fieldChanged(Field cdField, int context) {

            if (cdField instanceof CheckboxField) {
                CheckboxField cd = (CheckboxField) cdField;

                String curr = field.getText();
                if (cd.getChecked()) {
                    if (curr.endsWith("-shared")) {
                        // Do nothing
                    } else {
                        curr = curr + "-shared";
                    }
                } else {
                    if (curr.endsWith("-shared")) {
                        curr = curr.substring(0, curr.length() - "-shared".length());
                        controller.getDialogController().showMessage(
                                localization.getLanguage("custom_message_remove-cd"));
                    } else {
                        // Do nothing
                    }
                }
                field.setText(curr);
            }
            listener.fieldChanged(field, context);
        }
    }
}
