/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.blackberry.ui;

import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.ScreenDisplayController;
import com.funambol.blackberry.BBCustomization;

import com.funambol.util.Log;

public class SplashScreenHandler extends Thread {

    private static final String TAG_LOG = "SplashScreenHandler";

    private int delay;
    private Controller controller;
    private SplashScreen splashScreen = null;
    private SyncClient syncClient;

    public SplashScreenHandler(int delay, Controller controller, SyncClient syncClient) {
        this.delay = delay;
        this.controller = controller;
        this.syncClient = syncClient;
    }

    public void show() {
        ScreenDisplayController dc = controller.getScreenDisplayController();
        try {
            String splashScreenClassName = BBCustomization.getInstance().getSplashScreenClassName();
            // The splash screen is not mandatory, if not available we just
            // continue
            if (splashScreenClassName != null) {
                Class splashScreenClass = Class.forName(splashScreenClassName);
                splashScreen = (SplashScreen)splashScreenClass.newInstance();
                splashScreen.setHandler(this);
                dc.showScreen(splashScreen, true);
            } else {
                close();
            }
        } catch (Throwable e) {
            Log.error(TAG_LOG, "Cannot show splash screen", e);
            close();
            splashScreen = null;
        }
    }

    public void close() {
        if (splashScreen != null) {
            try {
                ScreenDisplayController dc = controller.getScreenDisplayController();
                dc.hideScreen(splashScreen);
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot hide splash screen", e);
            }
        }
        syncClient.showMainScreen();
    }

    public SplashScreen getSplashScreen() {
        return splashScreen;
    }
}


