/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import java.util.Hashtable;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.EncodedImage;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Field;

import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.BBLocalization;
import com.funambol.blackberry.ui.UISyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;

import com.funambol.client.source.AppSyncSource;
import com.funambol.client.controller.SyncingAnimation;
import com.funambol.util.Log;

public class RimAloneButtonSyncSource extends RimButtonSyncSource {

    // Point types array for rounded rectangle. Each point type
    // corresponds to one of the colors in the colors array. The
    // space marks the division between points on the top half of
    // the rectangle and those on the bottom.
    private static final byte[] PATH_POINT_TYPES = {
        Graphics.CURVEDPATH_END_POINT, 
        Graphics.CURVEDPATH_QUADRATIC_BEZIER_CONTROL_POINT,
        Graphics.CURVEDPATH_END_POINT, 
        Graphics.CURVEDPATH_END_POINT, 
        Graphics.CURVEDPATH_QUADRATIC_BEZIER_CONTROL_POINT,
        Graphics.CURVEDPATH_END_POINT, 
        Graphics.CURVEDPATH_END_POINT, 
        Graphics.CURVEDPATH_QUADRATIC_BEZIER_CONTROL_POINT,
        Graphics.CURVEDPATH_END_POINT, 
        Graphics.CURVEDPATH_END_POINT, 
        Graphics.CURVEDPATH_QUADRATIC_BEZIER_CONTROL_POINT,
        Graphics.CURVEDPATH_END_POINT, 
    };

    // Colors array for rounded rectangle gradient. Each color corresponds
    // to one of the points in the point types array. Top light, bottom black.
    private static final int[] PATH_GRADIENT = {
        0xF8F8F8, 0xF8F8F8, 0xF8F8F8, 0xF8F8F8, 0xF8F8F8, 0xF8F8F8,
        0xE0E0E0, 0xE0E0E0, 0xE0E0E0, 0xE0E0E0, 0xE0E0E0, 0xE0E0E0
    };

    private static final int BORDER_COLOR = 0xD0D0D0; // dark gray
    private static final int BACKGROUND_COLOR = 0xFFFFFF; // White

    private static final int CURVE_X = 12; // X-axis inset of curve
    private static final int CURVE_Y = 12; // Y-axis inset of curve

    protected SyncingAnimation animation = null;

    protected int buttonX;
    protected int buttonY;
    protected int buttonWidth;
    protected int buttonHeight;
    protected int absoluteY;

    protected Hashtable bitmapsMap = new Hashtable();
    protected com.funambol.client.ui.Bitmap currentArrow = null;

    protected BBCustomization customization = BBCustomization.getInstance();
    protected BBLocalization  localization  = BBLocalization.getInstance();

    public RimAloneButtonSyncSource()
    {
        super(null, null, null, null, null, null);
    }

    public RimAloneButtonSyncSource(String title, String status, String imageName,
                                    String imageDisabledName, int absoluteY)
    {
        super(title, status, imageName, imageDisabledName, null, null);
        this.absoluteY = absoluteY;
    }

    public void setOffsetY(int absoluteY) {
        this.absoluteY = absoluteY;
    }

    public void draw(Graphics graphics, int x, int y, int width, int height) {

        // Let's draw the background of the button. We use a percentage of the
        // available space for this and we have a different behavior for
        // landscape and portrait devices
        boolean landscape = width > height;

        if (landscape) {
            buttonWidth = (width * 6) / 10;
            buttonHeight = (height * 5) / 10;
        } else {
            buttonWidth = (width * 7) / 10;
            buttonHeight = (height * 55) / 100;
        }
        buttonX = (width - buttonWidth) / 2;
        buttonY = height / 10;

        // Clear the background (white)
        graphics.setColor(0xFFFFFF);
        graphics.fillRect(0,0,width - 1, height - 1);

        // Draw the button 
        graphics.translate(buttonX, buttonY);
        drawButtonBackground(graphics, buttonWidth, buttonHeight);
        int titleHeight = drawTitleText(graphics, buttonWidth, buttonHeight);
        drawArrows(graphics, buttonWidth, buttonHeight - titleHeight);
        graphics.translate(-buttonX, -buttonY);
        // Draw the status
        int statusY = drawStatusText(graphics, width, height);
        // Draw the source icon
        drawSourceIcon(graphics, width, height, statusY);
    }

    public void syncStarted() {
        // We have our own syncing animation in this case
        if (animation == null) {
            animation = new AloneSourceSyncingAnimation(false);
            animation.setAnimationIcons(customization.getStatusHugeIconsForAnimation());
        }
        animation.startAnimation();
    }

    public void syncEnded() {
        animation.stopAnimation();
    }

    public boolean isClickValid(int x, int y) {
        // The only valid clicks are the ones on the central buttons
        y -= absoluteY;
        if (x > buttonX && x < buttonX + buttonWidth &&
            y > buttonY && y < buttonY + buttonHeight) {

            return true;
        } else {
            return false;
        }
    }

    // This method is invoked during object construction, but we perform the
    // resize later, once we know better about the available space
    protected Bitmap resizeSourceIcon(String bitmapName) {
        return null;
    }

    protected void drawButtonBackground(Graphics g, int width, int height) {

        g.setGlobalAlpha(255);
        g.setBackgroundColor(BACKGROUND_COLOR);

        // Drawing within our margin.
        //int width = getPreferredWidth() - (MARGIN * 2);
        //int height = getPreferredHeight() - (MARGIN * 2);

        // Compute paths for the rounded rectangle. The 1st point (0) is on the left
        // side, right where the curve in the top left corner starts. So the top left
        // corner is point 1. These points correspond to our static arrays.
        int[] xPts = {
            0, 0, CURVE_X, width - CURVE_X, width, width,
            width, width, width - CURVE_X, CURVE_X, 0, 0
        };
        int[] yPts = {
            CURVE_Y, 0, 0, 0, 0, CURVE_Y,
            height - CURVE_Y, height, height, height, height, height - CURVE_Y
        };
        // Draw the gradient fill.
        g.drawShadedFilledPath(xPts, yPts, PATH_POINT_TYPES, PATH_GRADIENT, null);
        // Draw a rounded rectangle for the outline.
        // I think that drawRoundRect looks better than drawPathOutline.
        g.setColor(BORDER_COLOR);
        g.drawRoundRect(0, 0, width, height, CURVE_X * 2, CURVE_Y * 2);
        g.drawRoundRect(1, 1, width - 2, height - 2, CURVE_X * 2, CURVE_Y * 2);
    }

    protected int drawTitleText(Graphics graphics, int width, int height) {
        Font origFont = graphics.getFont();

        // The font height is set to 10% of the available space
        int fontHeight = height / 10;
        if (fontHeight < origFont.getHeight()) {
            fontHeight = origFont.getHeight();
        }
        Font f = font.derive(Font.BOLD, fontHeight);
        graphics.setFont(f);
        graphics.setColor(titleEnabledColor);

        int y = height - f.getHeight() - 10;

        String text = localization.getLanguage("home_sync_prefix") + " " + title;
        // Center the title in the button
        int x = (width - f.getAdvance(text)) / 2;
        graphics.drawText(text, x, y, DrawStyle.LEFT | DrawStyle.TOP, width - 10);

        graphics.setFont(origFont);

        // Return the vertical space occupied by this text
        return height - y;
    }

    protected Bitmap resizeArrow(String bitmapName, int width, int height) {
        int desiredWidth = (width * 9) / 10;
        int desiredHeight = (height * 9) / 10;
        int desiredSize = desiredWidth < desiredHeight ? desiredWidth : desiredHeight;

        Bitmap res = null;
        EncodedImage ei = EncodedImage.getEncodedImageResource(bitmapName);
        if (ei != null) {
            int bmpWidth = ei.getWidth();
            int bmpHeight = ei.getHeight();
 
            // Use the stricter scale but use a single scale factor to preserve
            // the image ratio
            float scaleX = (float)desiredSize / (float)bmpWidth;
            float scaleY = (float)desiredSize / (float)bmpHeight;

            float scale = scaleX < scaleY ? scaleX : scaleY;
            int newWidth = (int)((float)bmpWidth * scale);
            int newHeight = (int)((float)bmpHeight * scale);

            if (newWidth > 0 && newHeight > 0) {
                res = RimHelper.resizeImage(ei, newWidth, newHeight).getBitmap();
            }
        }
        return res;
    }

    protected void drawArrows(Graphics graphics, int width, int height) {

        if (currentArrow == null) {
            // The arrows have not been initialized yet, do it here
            // Scale if necessary. We want the arrow to be squared and use as
            // much as possible of the available space
            com.funambol.client.ui.Bitmap arrows[] = customization.getStatusHugeIconsForAnimation();
            if (arrows.length > 0) {
                currentArrow = arrows[0];
            }
        }

        if (currentArrow != null) {

            String currentArrowName = (String)currentArrow.getOpaqueDescriptor();
            Bitmap bmp = (Bitmap)bitmapsMap.get(currentArrowName);
            if (bmp == null) {
                bmp = resizeArrow(currentArrowName, width, height);
                bitmapsMap.put(currentArrowName, bmp);
            }

            int imageWidth = bmp.getWidth();
            int imageHeight = bmp.getHeight();
            // Center the image
            int imageTop = (height - imageHeight) / 2;
            int imageLeft = (width - imageWidth) / 2;
            graphics.drawBitmap(imageLeft, imageTop, imageWidth, imageHeight, bmp, 0, 0);
        }
    }

    protected int drawStatusText(Graphics graphics, int width, int height) {
        graphics.setColor(enabledStatusColor);

        // This text is always centered and close to the screen bottom
        int x = (width - font.getAdvance(status)) / 2;
        int y = (height - graphics.getFont().getHeight() - 5);

        graphics.drawText(status, x, y, DrawStyle.LEFT | DrawStyle.TOP, width - x);

        return y;
    }

    protected void drawSourceIcon(Graphics graphics, int width, int height, int statusY) {
        // The available space is: statusY - (buttonY + buttonHeight)
        int availableY = statusY - (buttonY + buttonHeight);

        Bitmap img;
        // If there is not enough space, we don't show this icon
        if (availableY < 32) {
            return;
        } else {
            // Resize the image to an appropriate dimension (if resizing is
            // required)
            int desiredSize = (availableY * 6) / 10;
            if (desiredSize > 64) {
                desiredSize = 64;
            }
            if (enabled) {
                if (image == null) {
                    image = RimHelper.getAndResizeImage(imageName, desiredSize, desiredSize);
                }
                img = image;
            } else {
                if (imageDisabled == null) {
                    imageDisabled = RimHelper.getAndResizeImage(imageDisabledName, desiredSize, desiredSize);
                }
                img = imageDisabled;
            }
        }

        // The source icon is just above the status text. We need to resize the
        // icon to an appropriate size, depending on the screen resolution
        int imageWidth = img.getWidth();
        int imageHeight = img.getHeight();

        int imageTop = statusY - imageHeight - 10;
        int imageLeft = (width - imageWidth) / 2;

        graphics.drawBitmap(imageLeft, imageTop, imageWidth, imageHeight, img, 0, 0);
    }

    protected class AloneSourceSyncingAnimation extends SyncingAnimation {

        public AloneSourceSyncingAnimation(boolean reset) {
            super(reset);
        }

        protected void showBitmap(com.funambol.client.ui.Bitmap bitmap) {
            currentArrow = bitmap;
            redraw();
        }
    }
}

