/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.ObjectChoiceField;
import net.rim.device.api.ui.container.VerticalFieldManager;
import net.rim.device.api.ui.container.HorizontalFieldManager;

import com.funambol.blackberry.BBConfiguration;
import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.BBLocalization;

import com.funambol.client.ui.SettingsUIItem;
import com.funambol.client.configuration.Configuration;
import com.funambol.util.Log;


public class RimLogSetting extends VerticalFieldManager implements SettingsUIItem {

    private static final String TAG_LOG = "RimLogSetting";

    private LabelField logLevelLabel;
    private ObjectChoiceField logLevelField;
    private LogLevelChoice choices[];

    private boolean hasChanges = false;

    private BBCustomization customization;
    private BBConfiguration configuration;
    private BBLocalization  localization = BBLocalization.getInstance();

    public RimLogSetting(BBCustomization customization, BBConfiguration configuration) {
        super();
        this.customization = customization;
        this.configuration = configuration;

        HorizontalFieldManager hfm = new HorizontalFieldManager();

        choices = new LogLevelChoice[4];
        choices[0] = new LogLevelChoice(localization.getLanguage("conf_log-level-0"), Log.ERROR);
        choices[1] = new LogLevelChoice(localization.getLanguage("conf_log-level-1"), Log.INFO);
        choices[2] = new LogLevelChoice(localization.getLanguage("conf_log-level-2"), Log.DEBUG);
        choices[3] = new LogLevelChoice(localization.getLanguage("conf_log-level-3"), Log.TRACE);

        logLevelLabel = new LabelField(localization.getLanguage("conf_log-level"));
        logLevelLabel.setFont(getFont().derive(Font.BOLD));

        logLevelField = new ObjectChoiceField("", choices, 0);

        hfm.add(logLevelLabel);
        hfm.add(logLevelField);

        add(hfm);

        // Register all listeners
        ChangeListener cl = new ChangeListener();
        logLevelField.setChangeListener(cl);
    }

    public void saveSettings(Configuration c) {
        hasChanges = false;
    }

    public void loadSettings(Configuration c) {
        hasChanges = false;
    }

    public void setLogLevel(int logLevel) {
        // Update the UI
        Log.trace(TAG_LOG, "setLogLevel to: " + logLevel);
        for(int i=0;i<choices.length;++i) {
            LogLevelChoice choice = choices[i];
            if (choice.getId() == logLevel) {
                logLevelField.setSelectedIndex(i);
                hasChanges = false;
                break;
            }
        }
    }

    public int getLogLevel() {
        LogLevelChoice choice = (LogLevelChoice)logLevelField.getChoice(logLevelField.getSelectedIndex());
        return choice.getId();
    }

    public boolean hasChanges() {
        return hasChanges;
    }

    private class ChangeListener implements FieldChangeListener {

        public void fieldChanged(Field field, int context) {
            if (context != FieldChangeListener.PROGRAMMATIC) {
                hasChanges = true;
            }
        }
    }

    private class LogLevelChoice {
        private String label;
        private int    id;

        public LogLevelChoice(String label, int id) {
            this.label = label;
            this.id    = id;
        }

        public String toString() {
            return label;
        }

        public int getId() {
            return id;
        }
    }
}

 
