/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import com.funambol.blackberry.BBAppSyncSourceManager;
import com.funambol.blackberry.BBConfiguration;
import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.BBLocalization;
import com.funambol.blackberry.controller.BBLoginScreenController;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.UISyncSourceController;
import com.funambol.blackberry.source.BBAppSyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;
import com.funambol.blackberry.ui.rim.component.ColoredSeparatorField;
import com.funambol.blackberry.ui.rim.component.ExtendedPasswordField;
import com.funambol.blackberry.ui.rim.component.JustifiedVerticalFieldManager;
import com.funambol.blackberry.ui.rim.component.RoundedFieldManager;
import com.funambol.blackberry.ui.rim.component.SingleLineEditField;
import com.funambol.blackberry.ui.rim.component.SpacerField;
import com.funambol.client.controller.AccountScreenController;
import com.funambol.client.controller.SynchronizationController;
import com.funambol.client.ui.LoginScreen;
import com.funambol.client.ui.SynchronizationScreen;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.Log;

import net.rim.device.api.math.Fixed32;
import net.rim.device.api.system.Display;
import net.rim.device.api.system.EncodedImage;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;


public class RimLoginScreen extends MainScreen implements LoginScreen,
        UISyncSourceContainer, SynchronizationScreen {
    
    private static final String TAG_LOG = "RimLoginScreen";

    private ButtonField loginButton = null;

    private SingleLineEditField   userEditField    = null;
    private SingleLineEditField   serverEditField  = null;
    private ExtendedPasswordField passEditField    = null;

    private Field firstFieldWithFocus = null;

    private VerticalFieldManager          bodyManagerNoFooter = null;
    private JustifiedVerticalFieldManager bodyManagerFooter   = null;

    private boolean loginEnabled = true;
    private boolean quitEnabled  = true;

    private BBConfiguration configuration = BBConfiguration.getInstance();
    private BBCustomization customization = BBCustomization.getInstance();
    private BBLocalization  localization  = BBLocalization.getInstance();

    private AccountScreenController screenController = null;
    

    private final Controller controller;

    private final MenuItem loginMenu = new MenuItem(BBLocalization.getInstance()
            .getLanguage("menu_login"), 20000, 10) {
            public void run() {
                saveConfiguration();
            }
    };

    private final MenuItem quitMenu  = new MenuItem(BBLocalization.getInstance()
            .getLanguage("menu_cancel"), 20000, 10) {
            public void run() {
                screenController.hide();
            }
    };

    private Font bigFont, mediumFont;

    public RimLoginScreen(Controller ctrl) {
        super(Field.USE_ALL_WIDTH | Field.USE_ALL_HEIGHT | 
                Field.FIELD_LEFT | Field.FIELD_TOP |
                Manager.VERTICAL_SCROLLBAR | Manager.VERTICAL_SCROLL);

        controller = ctrl;

        // Init fonts
        Font defaultFont = RimHelper.getDefaultFont();
        bigFont = defaultFont;
        mediumFont = defaultFont.derive(defaultFont.getStyle(),
                (int)(defaultFont.getHeight()*0.8));
        if(isSmallWidthDevice()) {
            defaultFont = mediumFont;
        }
        this.setFont(defaultFont);

        // Init spacers size
        int smallSpacerHeight, mediumSpacerHeight, bigSpacerHeight;
        int defaultSpacerWidth, smallSpacerWidth;
        int bodyContentPadding;
        if(isSmallHeightDevice()) {
            smallSpacerHeight = 3;
            mediumSpacerHeight = 6;
            bigSpacerHeight = 10;
        } else {
            smallSpacerHeight = 5;
            mediumSpacerHeight = 9;
            bigSpacerHeight = 14;
        }
        if(isVerySmallWidthDevice()) {
            defaultSpacerWidth = 5;
            smallSpacerWidth = 2;
            bodyContentPadding = 3;
        } else if(isSmallWidthDevice()) {
            defaultSpacerWidth = 5;
            smallSpacerWidth = 5;
            bodyContentPadding = 5;
        } else {
            defaultSpacerWidth = 5;
            smallSpacerWidth = 5;
            bodyContentPadding = 10;
        }

        // Init title field
        LabelField title = new LabelField(customization.getApplicationTitle());
        title.setFont(getFont().derive(RimTheme.TITLE_FONT_STYLE,
                RimTheme.TITLE_FONT_SIZE));
        setTitle(title);

        // Init signup image  field
        EncodedImage topLogo;
        if(isSmallHeightDevice()) {
            topLogo = EncodedImage.getEncodedImageResource((String)customization
                .getLoginLogoSmall().getOpaqueDescriptor());
        } else {
            topLogo = EncodedImage.getEncodedImageResource((String)customization
                .getLoginLogo().getOpaqueDescriptor());
        }
        if(topLogo.getWidth() > Display.getWidth()) {
            topLogo = scaleImageToWidth(topLogo, Display.getWidth());
        }
        BitmapField topImageField = new BitmapField(topLogo.getBitmap(),
                Field.FIELD_HCENTER | Field.FIELD_TOP | Field.NON_FOCUSABLE);
        LabelField topLabelField = new LabelField(
                localization.getLanguage("login_help_text"), Field.FIELD_LEFT);
        topLabelField.setFont(mediumFont);

        // Init server url field
        LabelField serverUriLabel = new LabelField(localization.getLanguage("conf_server-url"));
        serverUriLabel.setFont(getFont().derive(Font.BOLD));
        serverEditField = new SingleLineEditField();
        RoundedFieldManager serverUrlFieldContainer = null;
        serverUrlFieldContainer = new RoundedFieldManager(Field.USE_ALL_WIDTH);
        serverUrlFieldContainer.setFrameColor(RimTheme.DEFAULT_COLOR);
        serverUrlFieldContainer.setDrawShading(true);
        serverUrlFieldContainer.add(serverEditField);

        // Init username field
        LabelField usernameLabel = new LabelField(localization.getLanguage("conf_user"));
        usernameLabel.setFont(defaultFont.derive(Font.BOLD));
        userEditField = new SingleLineEditField();
        RoundedFieldManager userEditFieldContainer = new RoundedFieldManager(Field.USE_ALL_WIDTH);
        userEditFieldContainer.setFrameColor(RimTheme.DEFAULT_COLOR);
        userEditFieldContainer.setDrawShading(true);
        userEditFieldContainer.add(userEditField);

        // Init password field
        LabelField passwordLabel = new LabelField(localization.getLanguage("conf_password"));
        passwordLabel.setFont(defaultFont.derive(Font.BOLD));
        passEditField = new ExtendedPasswordField("",
                localization.getLanguage("conf_show_password"));

        int estimatedUserWidth = usernameLabel.getFont().getAdvance(usernameLabel.getText());
        int estimatedPassWidth = passwordLabel.getFont().getAdvance(passwordLabel.getText());
        int estimatedServerWidth = serverUriLabel.getFont().getAdvance(serverUriLabel.getText());
        int maxLabelWidth = Math.max(Math.max(estimatedUserWidth,
                estimatedPassWidth), estimatedServerWidth);
        if(maxLabelWidth == estimatedUserWidth) {
            estimatedPassWidth = maxLabelWidth-estimatedPassWidth;
            estimatedServerWidth = maxLabelWidth-estimatedServerWidth;
            estimatedUserWidth = 0;
        } else if(maxLabelWidth == estimatedPassWidth) {
            estimatedUserWidth = maxLabelWidth-estimatedUserWidth;
            estimatedServerWidth = maxLabelWidth-estimatedServerWidth;
            estimatedPassWidth = 0;
        } else if(maxLabelWidth == estimatedServerWidth) {
            estimatedUserWidth = maxLabelWidth-estimatedUserWidth;
            estimatedPassWidth = maxLabelWidth-estimatedPassWidth;
            estimatedServerWidth = 0;
        }

        HorizontalFieldManager usernameField =
                new HorizontalFieldManager(Field.USE_ALL_WIDTH);
        VerticalFieldManager usernameLabelContainer = new VerticalFieldManager();
        usernameLabelContainer.add(SpacerField.createVerticalSpacer(4));
        usernameLabelContainer.add(usernameLabel);
        usernameField.add(usernameLabelContainer);
        usernameField.add(SpacerField.createHorizontalSpacer(
                defaultSpacerWidth + estimatedUserWidth));
        usernameField.add(userEditFieldContainer);

        HorizontalFieldManager passwordField =
                new HorizontalFieldManager(Field.USE_ALL_WIDTH);
        VerticalFieldManager passwordLabelContainer = new VerticalFieldManager();
        passwordLabelContainer.add(SpacerField.createVerticalSpacer(4));
        passwordLabelContainer.add(passwordLabel);
        passwordField.add(passwordLabelContainer);
        passwordField.add(SpacerField.createHorizontalSpacer(
                defaultSpacerWidth + estimatedPassWidth));
        passwordField.add(passEditField);

        HorizontalFieldManager serverUrlField =
                new HorizontalFieldManager(Field.USE_ALL_WIDTH);
        VerticalFieldManager serverUrlLabelContainer = new VerticalFieldManager();
        serverUrlLabelContainer.add(SpacerField.createVerticalSpacer(4));
        serverUrlLabelContainer.add(serverUriLabel);
        serverUrlField.add(serverUrlLabelContainer);
        serverUrlField.add(SpacerField.createHorizontalSpacer(
                defaultSpacerWidth + estimatedServerWidth));
        serverUrlField.add(serverUrlFieldContainer);

        // Init login button
        loginButton = new ButtonField(localization.getLanguage("login_button"),
                Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
        loginButton.setFont(bigFont);
        loginButton.setChangeListener(new FieldChangeListener() {
            public void fieldChanged(Field field, int i) {
                saveConfiguration();
            }
        });

        ExtendedVerticalFieldManager bodyFields =
                new ExtendedVerticalFieldManager(
                bodyContentPadding, Field.USE_ALL_WIDTH);
        bodyFields.add(SpacerField.createVerticalSpacer(bodyContentPadding));
        bodyFields.add(topLabelField);
        bodyFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight +
                smallSpacerHeight));
        bodyFields.add(usernameField);
        bodyFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight));
        bodyFields.add(passwordField);
        bodyFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight));
        if (BBCustomization.getInstance().syncUriEditable()) {
            bodyFields.add(serverUrlField);
            bodyFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight +
                    mediumSpacerHeight));
        }
        bodyFields.add(loginButton);
        bodyFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight + smallSpacerHeight));

        VerticalFieldManager headerManager = new VerticalFieldManager(Field.USE_ALL_WIDTH |
                Field.FIELD_HCENTER);
        if(isSmallHeightDevice()) {
            headerManager.add(SpacerField.createVerticalSpacer(mediumSpacerHeight));
        } else {
            headerManager.add(SpacerField.createVerticalSpacer(smallSpacerHeight));
        }
        headerManager.add(topImageField);
        if(isSmallHeightDevice()) {
            headerManager.add(SpacerField.createVerticalSpacer(mediumSpacerHeight));
        } else {
            headerManager.add(SpacerField.createVerticalSpacer(smallSpacerHeight));
        }
        firstFieldWithFocus = createSeparatorField(true);
        headerManager.add(firstFieldWithFocus);

        HorizontalFieldManager bodyManager =
                new HorizontalFieldManager(Field.USE_ALL_WIDTH);
        bodyManager.add(SpacerField.createHorizontalSpacer(bodyContentPadding));
        bodyManager.add(bodyFields);

        bodyManagerNoFooter = new VerticalFieldManager(Field.FIELD_TOP);
        bodyManagerNoFooter.add(headerManager);
        bodyManagerNoFooter.add(bodyManager);

        if(customization.getMobileSignupEnabled() && !configuration.getSignupAccountCreated()) {
            // Init the footer fields containing signup feature
            HorizontalFieldManager footerContent = new HorizontalFieldManager(Field.FIELD_RIGHT);
            LabelField signupLabel = new LabelField(
                    localization.getLanguage("login_signup_label"),
                    Field.FIELD_VCENTER);
            if(isVerySmallWidthDevice()) {
                signupLabel.setFont(mediumFont);
            } else {
                signupLabel.setFont(defaultFont.derive(Font.BOLD));
            }
            ButtonField signupButton = new ButtonField(
                    localization.getLanguage("signup_button"),
                    ButtonField.CONSUME_CLICK);
            signupButton.setFont(mediumFont);
            signupButton.setChangeListener(new FieldChangeListener() {
                public void fieldChanged(Field field, int i) {
                    ((BBLoginScreenController)screenController).switchToSignupScreen();
                }
            });
            footerContent.add(signupLabel);
            footerContent.add(SpacerField.createHorizontalSpacer(smallSpacerWidth));
            footerContent.add(signupButton);
            footerContent.add(SpacerField.createHorizontalSpacer(smallSpacerWidth));

            VerticalFieldManager footerManager = new VerticalFieldManager();
            footerManager.add(createSeparatorField(false));
            footerManager.add(SpacerField.createVerticalSpacer(smallSpacerHeight));
            footerManager.add(footerContent);
            footerManager.add(SpacerField.createVerticalSpacer(smallSpacerHeight));

            bodyManagerFooter = new JustifiedVerticalFieldManager(
                    bodyManagerNoFooter, footerManager,
                    true, Field.USE_ALL_HEIGHT | Field.USE_ALL_WIDTH,
                    Display.getHeight() - /* Expected title height*/ 22);
            add(bodyManagerFooter);
        } else {
            add(bodyManagerNoFooter);
        }

        updateFocusedField();

        if (!BBCustomization.getInstance().userAccountEditable()) {
            usernameField.setEditable(false);
            passwordField.setEditable(false);
            if(serverEditField != null) {
                serverEditField.setEditable(false);
            }
        }

        initScreenController();

        screenController.initScreen();
        initialize(screenController);
    }

    public void initialize(AccountScreenController c) {
        BBAppSyncSource appSource = (BBAppSyncSource)
                BBAppSyncSourceManager.getInstance().getSource(
                BBAppSyncSourceManager.SYNC_FLAG_CONFIG);
        UISyncSourceController itemController = new UISyncSourceController(
                                                 controller, appSource);
        SyncSource ss = appSource.getSyncSource();
        ss.setListener(itemController);
        appSource.setClientUISyncSourceController(itemController);
    }

    public SynchronizationController getSynchronizationController() {
        return screenController;
    }

    private boolean isSmallHeightDevice() {
        return (Display.getHeight() <= 320);
    }

    private boolean isSmallWidthDevice() {
        return (Display.getWidth() <= 320);
    }

    private boolean isVerySmallWidthDevice() {
        return (Display.getWidth() <= 240);
    }

    private SeparatorField createSeparatorField(boolean focusable) {
        ColoredSeparatorField separator = new ColoredSeparatorField(
                focusable ? Field.FOCUSABLE : 0);
        separator.setColor(RimTheme.DEFAULT_COLOR);
        separator.setDrawShading(true);
        return separator;
    }

    private static EncodedImage scaleImageToWidth(EncodedImage encoded, int newWidth) {
        return scaleToFactor(encoded, encoded.getWidth(), newWidth);
    }

    private static EncodedImage scaleToFactor(EncodedImage encoded, int curSize, int newSize) {
        int numerator = Fixed32.toFP(curSize);
        int denominator = Fixed32.toFP(newSize);
        int scale = Fixed32.div(numerator, denominator);

        return encoded.scaleImage32(scale, scale);
    }

    protected void initScreenController() {
        screenController = new BBLoginScreenController(controller,
                BBCustomization.getInstance(),
                BBConfiguration.getInstance(),
                BBLocalization.getInstance(),
                BBAppSyncSourceManager.getInstance(),
                this);
        controller.setLoginScreenController(screenController);
    }

    /**
     * Invoked on user hitting the escape button. It prompts the user with a
     * dialog, if he wants to save/discard/cancel the changes in configuration
     *
     */
    public boolean onClose() {
        screenController.hide();
        return true;
    }

    /**
     * Invoked when a user selects the "Save" option from the menu in the
     * configuration screen or from the dialog when the user exits the
     * configuration screen.
     */
    public void saveConfiguration() {
        Log.debug(TAG_LOG, "saveConfiguration");

        String serverUri;
        if (BBCustomization.getInstance().syncUriEditable()) {
            serverUri = serverEditField.getText().trim();
        } else {
            serverUri = BBCustomization.getInstance().getServerUriDefault();
        }
        String username = userEditField.getText().trim();
        String password = passEditField.getText().trim();

        screenController.saveAndCheck(serverUri, username, password);
    }

    public void addShowPasswordField(boolean checked) {
    }

    public Object getUiScreen() {
        return this;
    }

    public void makeMenu(Menu menu, int instance) {
        if(loginEnabled) {
            menu.add(loginMenu);
        }
        if(quitEnabled) {
            menu.add(quitMenu);
        }
    }

    public void checkFailed() {
    }

    public void checkSucceeded() {
    }

    public void disableSave() {
        loginEnabled = false;
    }

    public void enableSave() {
        loginEnabled = true;
    }

    public void enableCancel() {
        quitEnabled = true;
    }

    public void disableCancel() {
        quitEnabled = false;
    }

    public void setSyncUrl(String url) {
        if (BBCustomization.getInstance().syncUriEditable()) {
            serverEditField.setText(url);
        }
    }

    public String getSyncUrl() {
        return serverEditField.getText();
    }

    public void setUsername(String username) {
        userEditField.setText(username);
    }

    public String getUsername() {
        return userEditField.getText();
    }

    public void setPassword(String password) {
        passEditField.setText(password);
    }

    public String getPassword() {
        return passEditField.getText();
    }

    public void redraw() {
        invalidate();
    }

    protected void onExposed() {
        super.onExposed();
        updateScreen();
    }

    protected void onDisplay() {
        super.onDisplay();
        updateScreen();
    }

    private void updateScreen() {
        updateFooterVisibility();
        if(screenController instanceof BBLoginScreenController &&
                !customization.getMobileSignupEnabled()) {
            ((BBLoginScreenController)screenController)
                    .showNetworkUsageWarningIfRequired();
        }
    }
    /**
     * Hides the footer if it was displayed and the signup account has been
     * already created.
     */
    private void updateFooterVisibility() {
        if(bodyManagerFooter != null && configuration.getSignupAccountCreated()) {
            bodyManagerFooter.delete(bodyManagerNoFooter);
            delete(bodyManagerFooter);
            bodyManagerFooter = null;
            add(bodyManagerNoFooter);
        }
    }

    private void updateFocusedField() {
        // Give the focus to the first focusabel field
        firstFieldWithFocus.setFocus();
    }

    private class ExtendedVerticalFieldManager extends VerticalFieldManager {
        private int paddingRight;

        public ExtendedVerticalFieldManager(int paddingRight, long style) {
            super(style);
            this.paddingRight = paddingRight;
        }

        protected void sublayout(int maxWidth, int maxHeight) {
            super.sublayout(maxWidth-paddingRight, maxHeight);
        }
    }
}
