/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import java.util.Vector;

import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.ObjectChoiceField;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.container.VerticalFieldManager;
import net.rim.device.api.ui.container.HorizontalFieldManager;

import com.funambol.blackberry.BBConfiguration;
import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.BBLocalization;

import com.funambol.platform.FileAdapter;
import com.funambol.client.source.AppSyncSource;
import com.funambol.client.ui.SettingsUIItem;
import com.funambol.client.source.AppSyncSourceConfig;
import com.funambol.client.configuration.Configuration;
import com.funambol.util.Log;


public class RimSyncModeSetting extends VerticalFieldManager implements SettingsUIItem {

    private static final String TAG_LOG = "RimSyncModeSettings";

    private LabelField syncModeLabel;
    private ObjectChoiceField syncModeField;
    private ObjectChoiceField pollingPIMIntervalField;
    private VerticalFieldManager vfm;
    private HorizontalFieldManager intervalHfm;
    private CheckboxField c2sPushField;

    private boolean hasChanges = false;

    private BBCustomization customization;
    private BBConfiguration configuration;
    private BBLocalization localization = BBLocalization.getInstance();

    public RimSyncModeSetting(BBCustomization customization, BBConfiguration configuration) {
        super();
        this.customization = customization;
        this.configuration = configuration;

        SyncModeChoice[] syncModes = getSyncModes();
        syncModeLabel = new LabelField(localization.getLanguage("conf_sync_mode-title"));
        syncModeLabel.setFont(getFont().derive(Font.BOLD));
        syncModeField = new ObjectChoiceField("", syncModes, 0);
        syncModeField.setFont(getFont());

        vfm = new VerticalFieldManager();
        HorizontalFieldManager hfm1 = new HorizontalFieldManager();
        hfm1.add(syncModeLabel);
        hfm1.add(syncModeField);
        vfm.add(hfm1);

        LabelField pollingPIMIntervalLabel = new LabelField(localization.
                                                     getLanguage("conf_polling-pim-duration"));
        pollingPIMIntervalLabel.setFont(getFont().derive(Font.BOLD));
        String[] choices = getIntervalChoices(customization.getPollingPimIntervalChoices());
        int index = getScheduledIntervalIndexFromMins(configuration.getPollingInterval());
        pollingPIMIntervalField = new ObjectChoiceField("", choices, index);
        pollingPIMIntervalField.setFont(getFont());

        intervalHfm = new HorizontalFieldManager();
        intervalHfm.add(pollingPIMIntervalLabel);
        intervalHfm.add(pollingPIMIntervalField);

        refreshSyncMode(syncModes);

        add(vfm);

        // Add the c2s push setting
        if (customization.isC2SPushEnabled()) {
            c2sPushField = new CheckboxField(localization.getLanguage("conf_c2s-push"), configuration.getC2SPush());
            add(c2sPushField);
        }

        // Register all listeners
        ChangeListener cl = new ChangeListener();
        syncModeField.setChangeListener(cl);
        pollingPIMIntervalField.setChangeListener(cl);
        if (c2sPushField != null) {
            c2sPushField.setChangeListener(cl);
        }
    }

    public void saveSettings(Configuration c) {
        if (syncModeField != null) {
            SyncModeChoice choice = (SyncModeChoice)syncModeField.getChoice(syncModeField.getSelectedIndex());
            int mode = choice.getId();
            configuration.setSyncMode(mode);
            if (mode == Configuration.SYNC_MODE_SCHEDULED && pollingPIMIntervalField != null) {
                int pollingIdx = pollingPIMIntervalField.getSelectedIndex();
                int pollingPeriod = getScheduledMinsFromIndex(pollingIdx);
                configuration.setPollingInterval(pollingPeriod);
            }
        }

        if (c2sPushField != null) {
            configuration.setC2SPush(c2sPushField.getChecked());
        }
        hasChanges = false;
    }

    public void loadSettings(Configuration c) {

        if (syncModeField != null) {
            refreshSyncMode(getSyncModes());
        }

        if (c2sPushField != null) {
            boolean c2sPush = configuration.getC2SPush();
            c2sPushField.setChecked(c2sPush);
        }

        hasChanges = false;
    }

    private void refreshSyncMode(SyncModeChoice syncModes[]) {
        // Set the sync mode from the configuration
        for(int i=0;i<syncModes.length;++i) {
            SyncModeChoice choice = syncModes[i];
            if (choice.getId() == configuration.getSyncMode()) {
                syncModeField.setSelectedIndex(i);
                if (choice.getId() == Configuration.SYNC_MODE_SCHEDULED &&
                    vfm.getFieldCount() == 1)
                {
                    vfm.add(intervalHfm);
                    // TODO: refresh the interval period
                } else if (vfm.getFieldCount() == 2) {
                    vfm.delete(intervalHfm);
                }
                break;
            }
        }

    }

    public boolean hasChanges() {
        return hasChanges;
    }

    private SyncModeChoice[] getSyncModes() {
        SyncModeChoice[] syncModes = null;

        boolean isPollEnabled =
                !(customization.disablePollOn8830() &&
                DeviceInfo.getDeviceName().equals("8830"));

        // At the moment we may have only manual and scheduled
        int size;
        if (isPollEnabled) {
            size = 2;
        } else {
            size = 1;
        }

        String manualOption = localization.getLanguage("conf_sync_mode-manual");
        String scheduledOption = localization.getLanguage("conf_sync_mode-scheduled");

        syncModes = new SyncModeChoice[size];
        syncModes[0] = new SyncModeChoice(manualOption, Configuration.SYNC_MODE_MANUAL);
        if (size >= 2) {
            syncModes[1] = new SyncModeChoice(scheduledOption, Configuration.SYNC_MODE_SCHEDULED);
        }

        return syncModes;
    }

    private int[] getScheduledIntervals() {
        return customization.getPollingPimIntervalChoices();
    }

    private String[] getIntervalChoices(int[] minValues) {
        int size = minValues.length;

        String minutes = localization.getLanguage("conf-mins");
        String hour = localization.getLanguage("conf-hour");
        String hours = localization.getLanguage("conf-hours");
        String day = localization.getLanguage("conf-day");
        String days = localization.getLanguage("conf-days");

        String[] intervalChoices = new String[size];

        for (int i = 0; i < size; i++) {
            int hoursValue = minValues[i]/60;

            if (hoursValue == 0) {
                intervalChoices[i] = minValues[i] + " " + minutes;
            } else if (hoursValue == 1) {
                intervalChoices[i] = hoursValue + " " + hour;
            } else if (hoursValue < 24) {
                intervalChoices[i] = hoursValue + " " + hours;
            } else if (hoursValue == 24){
                intervalChoices[i] = hoursValue/24 + " " + day;
            } else if (hoursValue > 24){
                intervalChoices[i] = hoursValue/24 + " " + day;
            }
        }

        return intervalChoices;
    }

    private int getScheduledIntervalIndexFromMins(int mins) {
        int[]    ints = getScheduledIntervals();
        // Backward search in the array
        for(int i=0;i<ints.length;++i) {
            int val = ints[i];
            if (val == mins) {
                return i;
            }
        }
        Log.error(TAG_LOG, "Invalid scheduling interval value " + mins);
        // We should never get here, but if we do, we just choose the first
        // value in the list to avoid exceptions
        return 0;
    }

    private int getScheduledMinsFromIndex(int idx) {
        int[]    ints = getScheduledIntervals();
        try {
            return ints[idx];
        } catch (ArrayIndexOutOfBoundsException e) {
            Log.error(TAG_LOG, "Invalid scheduling interval index " + idx);
            return 15;
        }
    }

    private class ChangeListener implements FieldChangeListener {

        public void fieldChanged(Field field, int context) {
            if (context != FieldChangeListener.PROGRAMMATIC) {
                hasChanges = true;
            }
            if (field == syncModeField) {
                // We may need to hide or show the scheduled interval
                SyncModeChoice choice = (SyncModeChoice)syncModeField.getChoice(syncModeField.getSelectedIndex());
                if (choice.getId() == Configuration.SYNC_MODE_SCHEDULED) {
                    if (vfm.getFieldCount() == 1) {
                        vfm.add(intervalHfm);
                    }
                } else {
                    if (vfm.getFieldCount() == 2) {
                        vfm.delete(intervalHfm);
                    }
                }
            }
        }
    }

    private class SyncModeChoice {
        private String label;
        private int    id;

        public SyncModeChoice(String label, int id) {
            this.label = label;
            this.id    = id;
        }

        public String toString() {
            return label;
        }

        public int getId() {
            return id;
        }
    }


}

 
