/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim.component;

import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.component.SeparatorField;

/**
 * A SeparatorField with an optional color
 */
public class ColoredSeparatorField extends SeparatorField {

    private int color = Color.BLACK;
    private boolean drawShading = false;

    public ColoredSeparatorField() {
        super();
    }

    public ColoredSeparatorField(long style) {
        super(style);
    }

    protected void paint(Graphics g) {
        int oldColor = g.getColor();
        if (color != -1) {
            g.setColor(color);
        }
        g.drawRect(0, 0, getWidth(), 2);
        if(drawShading) {
            g.setColor(Color.GRAY);
            g.drawRect(0, 2, getWidth(), 1);
            g.setColor(Color.LIGHTGREY);
            g.drawRect(0, 3, getWidth(), 1);
        }
        g.setColor(oldColor);
    }

    public int getPreferredHeight() {
        return 2 + (drawShading ? 2 : 0);
    }

    protected void layout(int w, int h) {
        setExtent(w, getPreferredHeight());
    }

    public void setColor(int c) {
        color = c;
    }

    public int getColor() {
        return color;
    }

    public void setDrawShading(boolean drawShading) {
        this.drawShading = drawShading;
    }
}
