/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim.component;

import com.funambol.blackberry.ui.rim.RimTheme;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.EditField;
import net.rim.device.api.ui.component.PasswordEditField;
import net.rim.device.api.ui.container.VerticalFieldManager;

/**
 * Defines an extended password edit field which allows also to add the
 * "Show password" checkbox in order to show ora hide the password text.
 */
public class ExtendedPasswordField extends VerticalFieldManager {

    private String password = null;
    private String showPasswordLabel = null;
    private SingleLineEditField editField = null;
    private CheckboxField  showPasswordField = null;
    private RoundedFieldManager roundedFieldManager = null;

    public ExtendedPasswordField(String psw, String showPasswordLabel) {
        super();
        this.password = psw;
        this.showPasswordLabel = showPasswordLabel;
        this.roundedFieldManager = new RoundedFieldManager(Field.USE_ALL_WIDTH);
        roundedFieldManager.setFrameColor(RimTheme.DEFAULT_COLOR);
        roundedFieldManager.setDrawShading(true);
        setPasswordVisible(false);
        add(roundedFieldManager); 
    }

    private void setPasswordVisible(boolean visible) {
        String psw = "";
        boolean editable = true;
        if(editField != null) {
            psw = editField.getText();
            editable = editField.isEditable();
            roundedFieldManager.delete(editField);
        } else {
            psw = password;
        }
        if(visible) {
            // Passing the NO_COMPLEX_INPUT disables the AutoText feature
            editField = new SingleLineEditField(EditField.NO_COMPLEX_INPUT);
            editField.setText(psw);
        } else {
            editField = new SingleLinePasswordField(psw);
        }
        editField.setEditable(editable);
        roundedFieldManager.add(editField);
    }

    public void addShowPasswordField(boolean checked, Font font) {
        if(showPasswordField == null) {
            showPasswordField = new CheckboxField(showPasswordLabel, checked);
            showPasswordField.setFont(font);
            showPasswordField.setChangeListener(new FieldChangeListener() {
                public void fieldChanged(Field field, int i) {
                    setPasswordVisible(showPasswordField.getChecked());
                }
            });
            add(showPasswordField);
            setPasswordVisible(checked);
        }
    }

    public String getText() {
        if(editField != null) {
            return editField.getText();
        }
        return password;
    }

    public void setText(String text) {
        password = text;
        if(editField != null) {
            editField.setText(text);
        }
    }

    public boolean isPasswordShowed() {
        return !(editField instanceof SingleLinePasswordField);
    }

    public void setEditable(boolean editable) {
        if(editField != null) {
            editField.setEditable(editable);
        }
    }

    public class SingleLinePasswordField extends SingleLineEditField {

        public SingleLinePasswordField(String password) {
            super();
            delete(editField);
            editField = new PasswordEditField();
            editField.setText(password);
            add(editField);
        }
    }
}

