/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim.component;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Manager;

/**
 * Represents a VerticalFieldManager which aligns two given fields vertically
 * and in justified mode in order to fille the entire screen height.
 */
public class JustifiedVerticalFieldManager extends Manager {
    
    private Field mTopField;
    private Field mBottomField;
    
    private int mAvailableHeight;
    
    public JustifiedVerticalFieldManager(Field topField, Field bottomField,
            boolean giveTopFieldPriority, long style, int availableHeight) {
        super(style);

        mTopField = topField;
        mBottomField = bottomField;

        add(topField);
        add(bottomField);

        mAvailableHeight = availableHeight;
    }
    
    protected void sublayout(int width, int height) {

        int consumedHeight = 0;
        if(mTopField != null) {
            // Layout top field
            layoutChild(mTopField, width - mTopField.getMarginLeft() -
                    mTopField.getMarginRight(), /* Shouldn't be higher */ 10000);
            consumedHeight += getTotalFieldHeight(mTopField);
        }
        if(mBottomField != null) {
            // Layout bottom field
            layoutChild(mBottomField, width - mBottomField.getMarginLeft() -
                    mBottomField.getMarginRight(), /* Shouldn't be higher */ 10000);
            consumedHeight += getTotalFieldHeight(mBottomField);
        }
        if(mTopField != null) {
            // Top field position
            setPositionChild(mTopField, getFieldX(mTopField, width),
                    mTopField.getMarginTop());
        }
        int bottomFieldTop;
        int extentHeight;
        if(consumedHeight < mAvailableHeight) {
            extentHeight = mAvailableHeight;
            bottomFieldTop = mAvailableHeight - getTotalFieldHeight(mBottomField);
        } else {
            extentHeight = consumedHeight;
            bottomFieldTop = consumedHeight-getTotalFieldHeight(mBottomField);
        }
        if(mBottomField != null) {
            // Bottom field position
            setPositionChild(mBottomField, getFieldX(mBottomField, width),
                    bottomFieldTop);
        }
        setExtent(width, extentHeight);
    }

    private int getTotalFieldHeight(Field f) {
        return f.getHeight() + f.getMarginTop() + f.getMarginBottom();
    }

    private int getFieldX(Field field, int width) {
        if(field.isStyle(Field.FIELD_RIGHT)) {
            return width - field.getWidth() - field.getMarginRight();
        } else if(field.isStyle(Field.FIELD_LEFT)) {
            return field.getMarginLeft() + (width - field.getMarginLeft() -
                    field.getWidth() - field.getMarginRight()) / 2;
        } else {
            return field.getMarginLeft();
        }
    }

    public void delete(Field field) {
        if(field == mTopField) {
            mTopField = null;
        } else if(field == mBottomField) {
            mBottomField = null;
        }
        super.delete(field);
    }
    
    public Field getTopField() {
        return mTopField;
    }
    
    public Field getBottomField() {
        return mBottomField;
    }
}    
