/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim.component;

import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.component.BitmapField;

/**
 * Defines a Manager which draws a rounded rectangle outside to first Field
 * contained in.
 */
public class RoundedFieldManager extends Manager {

    private static final int MARGIN_TOP    = 0;
    private static final int MARGIN_BOTTOM = 0;
    private static final int MARGIN_LEFT   = 0;
    private static final int MARGIN_RIGHT  = 0;

    private static final int DEFAULT_PADDING = 4;

    private static final int DEFAULT_ARC   = 15;

    private int frameColor = -1;
    private boolean drawShading = false;

    private int mPadding = DEFAULT_PADDING;
    private int mArc     = DEFAULT_ARC;

    private Field alignedField = null;

    public RoundedFieldManager(long style) {
        super(style);
    }
    
    protected void paint(Graphics g) {
        int oldColor = g.getColor();
        super.paint(g);
        // Now paint the border if required
        if (frameColor != -1 && getFieldCount() > 0) {
            g.setColor(frameColor);
            // We want a tick border
            g.drawRoundRect(MARGIN_LEFT, MARGIN_TOP,
                    getWidth() - (MARGIN_LEFT + MARGIN_RIGHT),
                    getHeight() - (MARGIN_TOP + MARGIN_BOTTOM),
                    mArc, mArc);
            if(drawShading) {
                g.setColor(Color.GRAY);
                g.drawRoundRect(MARGIN_LEFT + 1, MARGIN_TOP + 1,
                    getWidth() - (MARGIN_LEFT + MARGIN_RIGHT) - 2,
                    getHeight() - (MARGIN_TOP + MARGIN_BOTTOM) - 2,
                    mArc, mArc);
            }
        }
        g.setColor(oldColor);
    }

    protected void sublayout(int maxWidth, int maxHeight) {
        // Get max width from the given field if any
        if(alignedField != null) {
            int w = alignedField.getPreferredWidth();
            if(w > 0) {
                maxWidth = w;
            }
        }
        if(getFieldCount() > 0) {
            Field field = getField(0);
            layoutChild(field, maxWidth-getHorizontalEmptySpace(),
                    maxHeight-getVerticalEmptySpace());
            setPositionChild(field, MARGIN_LEFT + mPadding,
                    MARGIN_TOP + mPadding);
            setExtent( maxWidth - MARGIN_RIGHT, field.getHeight() + getVerticalEmptySpace());
        }
    }

    private int getHorizontalEmptySpace() {
        return MARGIN_LEFT + MARGIN_RIGHT + 2*mPadding;
    }

    private int getVerticalEmptySpace() {
        return MARGIN_TOP + MARGIN_BOTTOM + 2*mPadding;
    }

    public void setFrameColor(int c) {
        frameColor = c;
    }

    public int getFrameColor() {
        return frameColor;
    }

    public void setFramePadding(int padding) {
        mPadding = padding;
    }

    public void setFrameArc(int arc) {
        mArc = arc;
    }

    public void setDrawShading(boolean drawShading) {
        this.drawShading = drawShading;
    }

    public void alignToField(Field f) {
        alignedField = f;
    }
}
